﻿using System;
using Newtonsoft.Json;

namespace ECrash.Core.Models;

public class DateJsonConverter : JsonConverter
{
	public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
	{
		if (!(value is Date date))
		{
			writer.WriteNull();
		}
		else
		{
			writer.WriteValue(date.ToIsoString());
		}
	}

	public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
	{
		if (!(reader.Value is string s)) return null;
		var parts = s.Split('-');
		if (parts.Length != 3) throw new FormatException("Value must be in the format yyyy-MM-dd");
		return new Date(int.Parse(parts[0]), int.Parse(parts[1]), int.Parse(parts[2]));
	}

	public override bool CanConvert(Type objectType)
	{
		return objectType == typeof(Date) || objectType == typeof(Date?);
	}
}