﻿using System;
using System.Text;

namespace ECrash.Core.Models.Document;

public class Actor : IEquatable<Actor>, IUserIdentity
{
	public Actor(Guid userId, string username, string rank, string firstName, string middleName, string lastName, string nameSuffix, string officerId)
	{
		UserId = userId;
		Username = username;
		Rank = rank;
		FirstName = firstName;
		MiddleName = middleName;
		LastName = lastName;
		NameSuffix = nameSuffix;
		OfficerId = officerId;
	}

	public Guid UserId { get; }
	[MaxLength(100)] public string Username { get; }
	[MaxLength(25)] public string Rank { get; }
	[MaxLength(50)] public string FirstName { get; }
	[MaxLength(50)] public string MiddleName { get; }
	[MaxLength(50)] public string LastName { get; }
	[MaxLength(10)] public string NameSuffix { get; }
	[MaxLength(12)] public string OfficerId { get; }

	public override string ToString()
	{
		string separator = string.Empty;
		var sb = new StringBuilder();
		if (!string.IsNullOrWhiteSpace(Rank))
		{
			sb.Append(Rank);
			separator = " ";
		}
		if (!string.IsNullOrWhiteSpace(FirstName))
		{
			sb.Append(separator).Append(FirstName);
			separator = " ";
		}
		if (!string.IsNullOrWhiteSpace(MiddleName))
		{
			sb.Append(separator).Append(MiddleName);
			separator = " ";
		}
		if (!string.IsNullOrWhiteSpace(LastName))
		{
			sb.Append(separator).Append(LastName);
			separator = " ";
		}
		if (!string.IsNullOrWhiteSpace(NameSuffix))
		{
			sb.Append(separator).Append(NameSuffix);
			separator = " ";
		}
		if (!string.IsNullOrWhiteSpace(OfficerId))
		{
			sb.Append(separator).Append('(').Append(OfficerId).Append(')');
		}
		return sb.ToString();
	}

	public bool Equals(Actor other)
	{
		if (ReferenceEquals(null, other)) return false;
		if (ReferenceEquals(this, other)) return true;
		return UserId.Equals(other.UserId)
		       && string.Equals(Username, other.Username)
		       && string.Equals(Rank, other.Rank)
		       && string.Equals(FirstName, other.FirstName)
		       && string.Equals(MiddleName, other.MiddleName)
		       && string.Equals(LastName, other.LastName)
		       && string.Equals(NameSuffix, other.NameSuffix)
		       && string.Equals(OfficerId, other.OfficerId);
	}

	public override bool Equals(object obj)
	{
		if (ReferenceEquals(null, obj)) return false;
		if (ReferenceEquals(this, obj)) return true;
		if (obj.GetType() != this.GetType()) return false;
		return Equals((Actor)obj);
	}

	public override int GetHashCode()
	{
		unchecked
		{
			int hashCode = UserId.GetHashCode();
			hashCode = (hashCode * 397) ^ (Username != null ? Username.GetHashCode() : 0);
			hashCode = (hashCode * 397) ^ (Rank != null ? Rank.GetHashCode() : 0);
			hashCode = (hashCode * 397) ^ (FirstName != null ? FirstName.GetHashCode() : 0);
			hashCode = (hashCode * 397) ^ (MiddleName != null ? MiddleName.GetHashCode() : 0);
			hashCode = (hashCode * 397) ^ (LastName != null ? LastName.GetHashCode() : 0);
			hashCode = (hashCode * 397) ^ (NameSuffix != null ? NameSuffix.GetHashCode() : 0);
			hashCode = (hashCode * 397) ^ (OfficerId != null ? OfficerId.GetHashCode() : 0);
			return hashCode;
		}
	}

	public static bool operator ==(Actor left, Actor right)
	{
		return Equals(left, right);
	}

	public static bool operator !=(Actor left, Actor right)
	{
		return !Equals(left, right);
	}
}