﻿using System;

namespace ECrash.Core.Models.Document;

public struct AttachmentKey : IEquatable<AttachmentKey>
{
	public Guid Id { get; }
	public Filename Filename { get; }
	public Hash Hash { get; }

	public AttachmentKey(Guid id, Filename filename, Hash hash)
	{
		Id = id;
		Filename = filename;
		Hash = hash;
	}

	public bool Equals(AttachmentKey other)
	{
		return Id.Equals(other.Id)
		       && Filename.Equals(other.Filename)
		       && Hash.Equals(other.Hash);
	}

	public override bool Equals(object obj)
	{
		if (ReferenceEquals(null, obj)) return false;
		if (obj.GetType() != GetType()) return false;
		return Equals((AttachmentKey)obj);
	}

	public override int GetHashCode()
	{
		unchecked
		{
			int hashCode = Id.GetHashCode();
			hashCode = (hashCode * 397) ^ Filename.GetHashCode();
			hashCode = (hashCode * 397) ^ Hash.GetHashCode();
			return hashCode;
		}
	}

	public static bool operator ==(AttachmentKey left, AttachmentKey right)
	{
		return Equals(left, right);
	}

	public static bool operator !=(AttachmentKey left, AttachmentKey right)
	{
		return !Equals(left, right);
	}
}