﻿using ECrash.Core.Models.ReferenceData;

namespace ECrash.Core.Models.Document.BusinessRules;

public class DocumentBusinessRuleBase<TDocument> : IBusinessRule<TDocument> where TDocument : DocumentRoot<TDocument>
{
	protected DocumentBusinessRuleBase(IReferenceDataCache referenceDataCache)
	{
		ReferenceDataCache = referenceDataCache;
	}

	protected IReferenceDataCache ReferenceDataCache { get; }

	public void OnInitialize(TDocument document)
	{
		using (document.SuspendNotifications())
		{
			OnInitializeCore(document, GetFilteredReferenceData(document));
		}
	}

	public void OnPropertyChange(TDocument document, DocumentPropertyChangedEventArgs change)
	{
		OnPropertyChangeCore(document, GetFilteredReferenceData(document), change);
	}

	public void OnCollectionChange(TDocument document, DocumentCollectionChangedEventArgs change)
	{
		OnCollectionChangeCore(document, GetFilteredReferenceData(document), change);
	}

	public void OnNodeAttached(TDocument document, IDocumentNode<TDocument> node)
	{
		OnNodeAttached(document, node, GetFilteredReferenceData(document));
	}

	public void OnNodeDetached(TDocument document, IDocumentNode<TDocument> node)
	{
		OnNodeDetached(document, node, GetFilteredReferenceData(document));
	}

	protected ReferenceDataFilter GetFilteredReferenceData(TDocument document)
	{
		return ReferenceDataCache.GetFiltered(document.ReferenceDataEpoch);
	}

	protected virtual void OnInitializeCore(TDocument document, ReferenceDataFilter referenceData)
	{
	}

	protected virtual void OnPropertyChangeCore(TDocument document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
	{
	}

	protected virtual void OnCollectionChangeCore(TDocument document, ReferenceDataFilter referenceData, DocumentCollectionChangedEventArgs change)
	{
	}

	protected virtual void OnNodeAttached(TDocument document, IDocumentNode<TDocument> node, ReferenceDataFilter referenceData)
	{
	}

	protected virtual void OnNodeDetached(TDocument document, IDocumentNode<TDocument> node, ReferenceDataFilter referenceData)
	{
	}
}