﻿using ECrash.Core.Models.ReferenceData;

namespace ECrash.Core.Models.Document.BusinessRules;

public abstract class DocumentChildBusinessRuleBase<TDocument, TModel> : DocumentBusinessRuleBase<TDocument>
	where TDocument : DocumentRoot<TDocument>
	where TModel : IDocumentNode<TDocument>
{
	private readonly string[] _properties;

	protected DocumentChildBusinessRuleBase(IReferenceDataCache referenceDataCache, params string[] properties) : base(referenceDataCache)
	{
		_properties = properties;
	}

	protected override void OnPropertyChangeCore(TDocument document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
	{
		if (ShouldApplyRule(change, out var model))
		{
			ApplyRule(model, referenceData);
		}
	}

	protected override void OnNodeAttached(TDocument document, IDocumentNode<TDocument> node, ReferenceDataFilter referenceData)
	{
		if (node is TModel model)
		{
			ApplyRule(model, referenceData);
		}
	}

	protected virtual bool ShouldApplyRule(DocumentPropertyChangedEventArgs change, out TModel child)
	{
		if (change.Source is TModel model)
		{
			child = model;
			return change.IsForAny(_properties);
		}
		child = default;
		return false;
	}

	protected abstract void ApplyRule(TModel model, ReferenceDataFilter referenceData);
}