﻿using ECrash.Core.Models.ReferenceData;

namespace ECrash.Core.Models.Document.BusinessRules;

public abstract class DocumentRootBusinessRuleBase<TDocument> : DocumentBusinessRuleBase<TDocument>
	where TDocument : DocumentRoot<TDocument>
{
	private readonly string[] _properties;

	protected DocumentRootBusinessRuleBase(IReferenceDataCache referenceDataCache, params string[] properties) : base(referenceDataCache)
	{
		_properties = properties;
	}

	protected override void OnPropertyChangeCore(TDocument document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
	{
		if (ShouldApplyRule(change))
		{
			ApplyRule(document, referenceData, change);
		}
	}

	protected virtual bool ShouldApplyRule(DocumentPropertyChangedEventArgs change)
	{
		return change.Source is TDocument && change.IsForAny(_properties);
	}

	protected abstract void ApplyRule(TDocument document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change);
}