﻿using ECrash.Core.Models.ReferenceData;

namespace ECrash.Core.Models.Document.BusinessRules;

public abstract class DocumentValueCollectionBusinessRuleBase<TDocument, TModel, TCollection, TValue> : DocumentBusinessRuleBase<TDocument>
	where TDocument : DocumentRoot<TDocument>
	where TModel : IDocumentNode<TDocument>
	where TCollection : DocumentValueCollection<TDocument, TValue>
{
	protected DocumentValueCollectionBusinessRuleBase(IReferenceDataCache referenceDataCache) : base(referenceDataCache)
	{
	}

	protected override void OnCollectionChangeCore(TDocument document, ReferenceDataFilter referenceData, DocumentCollectionChangedEventArgs change)
	{
		if (change.Source is TCollection collection && collection.Parent is TModel parent && ShouldApplyRule(parent, collection))
		{
			ApplyRule(parent, collection, referenceData);
		}
	}

	protected virtual bool ShouldApplyRule(TModel model, TCollection collection)
	{
		return true;
	}

	protected abstract void ApplyRule(TModel model, TCollection collection, ReferenceDataFilter referenceData);
}