﻿using System;
using System.Collections.Generic;
using Caliburn.Micro;

namespace ECrash.Core.Models.Document;

public class DocumentChildCollection<TRoot, TChild> : DocumentValueCollectionWithoutDuplicates<TRoot, TChild>
	where TRoot : DocumentRoot<TRoot>
	where TChild : DocumentChild<TRoot>
{
	public override IEnumerable<IDocumentNode<TRoot>> EnumerateChildNodes()
	{
		return Items;
	}

	protected override void InsertItemBase(int index, TChild item)
	{
		if (item == null) throw new ArgumentNullException(nameof(item), "DocumentChildCollection does not allow null items");
		item.BeginUpdateLinks(this);
		base.InsertItemBase(index, item);
		item.EndUpdateLinks(this);
		UpdateIndexes();
	}

	protected override void RemoveItemBase(int index)
	{
		var oldItem = this[index];
		base.RemoveItemBase(index);
		oldItem.UpdateLinks(null);
		UpdateIndexes();
	}

	protected override void SetItemBase(int index, TChild item)
	{
		if (item == null) throw new ArgumentNullException(nameof(item), "DocumentChildCollection does not allow null items");
		TChild oldItem = this[index];
		oldItem?.BeginUpdateLinks(null);
		item.BeginUpdateLinks(this);
		base.SetItemBase(index, item);
		item.EndUpdateLinks(this);
		oldItem?.EndUpdateLinks(null);
		UpdateIndexes();
	}

	protected override void ClearItemsBase()
	{
		this.Apply(item =>
		{
			item.UpdateLinks(null);
		});
		base.ClearItemsBase();
		UpdateIndexes();
	}

	protected override void MoveItem(int oldIndex, int newIndex)
	{
		base.MoveItem(oldIndex, newIndex);
		UpdateIndexes();
	}

	protected void UpdateIndexes()
	{
		foreach (var item in Items)
		{
			var needsCollectionIndex = item as INeedsCollectionIndex;
			needsCollectionIndex?.OnCollectionIndexChanged();
		}
	}
}