﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;

namespace ECrash.Core.Models.Document;

[JsonObject(MemberSerialization.OptOut)]
public abstract class DocumentStatusBase<TApprovalStatus> : ExtendedPropertyChangedBase
	where TApprovalStatus : Enum
{
	private int _latestRevision;
	private string _localCaseNumber;
	private string _stateCaseNumber;
	private Date? _date;
	private Time? _time;
	private TApprovalStatus _approvalStatus;
	private DateTime _approvalStatusChangeTimestamp;
	private string _approvalMessage;
	private Actor _owner;
	private Actor _reviewer;
	private DateTime? _uploadTimestamp;
	private string _creatorSignature;
	private DateTime? _creatorSignatureTimestamp;

	protected DocumentStatusBase(Guid id)
	{
		Id = id;
	}

	public Guid Id { get; }

	public int LatestRevision
	{
		get => _latestRevision;
		set => Set(ref _latestRevision, value);
	}

	public string LocalCaseNumber
	{
		get => _localCaseNumber;
		set => Set(ref _localCaseNumber, value);
	}

	public string StateCaseNumber
	{
		get => _stateCaseNumber;
		set => Set(ref _stateCaseNumber, value);
	}

	public Date? Date
	{
		get => _date;
		set => SetWithDependencies(ref _date, value, nameof(Date), nameof(DateTime));
	}

	public Time? Time
	{
		get => _time;
		set => SetWithDependencies(ref _time, value, nameof(Time), nameof(DateTime));
	}

	public DateTime? DateTime => _date.Combine(_time);

	public TApprovalStatus ApprovalStatus
	{
		get => _approvalStatus;
		set => Set(ref _approvalStatus, value);
	}

	public DateTime ApprovalStatusChangeTimestamp
	{
		get => _approvalStatusChangeTimestamp;
		set => Set(ref _approvalStatusChangeTimestamp, value);
	}

	public string ApprovalMessage
	{
		get => _approvalMessage;
		set => Set(ref _approvalMessage, value);
	}

	public Actor Owner
	{
		get => _owner;
		set => Set(ref _owner, value);
	}

	public ActorList PreviousOwners { get; } = new ActorList();

	public Actor Reviewer
	{
		get => _reviewer;
		set => Set(ref _reviewer, value);
	}

	public DateTime? UploadTimestamp
	{
		get => _uploadTimestamp;
		set => Set(ref _uploadTimestamp, value);
	}

	public string CreatorSignature
	{
		get => _creatorSignature;
		set => Set(ref _creatorSignature, value);
	}

	public DateTime? CreatorSignatureTimestamp
	{
		get => _creatorSignatureTimestamp;
		set => Set(ref _creatorSignatureTimestamp, value);
	}

	public ReviewPropertyNoteList ReviewPropertyNotes { get; } = new();

	public abstract bool IsStateCaseNumberValid();
}