﻿using System;
using ECrash.Core.Models.ReferenceData;

namespace ECrash.Core.Models.Document;

public class DocumentValueCollectionWithoutDuplicates<TRoot, TValue> : DocumentValueCollection<TRoot, TValue> 
	where TRoot : DocumentRoot<TRoot>
{
	protected override void InsertItemBase(int index, TValue item)
	{
		if (Contains(item))
		{
			// Ignore duplicate reference data values, but throw on other duplicates
			if (item is not ReferenceDataBase)
			{
				throw new InvalidOperationException("Duplicate values are not allowed.");
			}
			return;
		}
		base.InsertItemBase(index, item);
	}

	protected override void SetItemBase(int index, TValue item)
	{
		int existingIndex = IndexOf(item);
		if (existingIndex >= 0 && existingIndex != index)
		{
			// Ignore duplicate reference data values, but throw on other duplicates
			if (item is not ReferenceDataBase)
			{
				throw new InvalidOperationException("Duplicate values are not allowed.");
			}
			return;
		}
		base.SetItemBase(index, item);
	}
}