﻿using System.Runtime.CompilerServices;
using Caliburn.Micro;
using JetBrains.Annotations;

namespace ECrash.Core.Models.Document;

public class ExtendedPropertyChangedBase : PropertyChangedBase
{
	[NotifyPropertyChangedInvocator]
	public override bool Set<T>(ref T oldValue, T newValue, [CallerMemberName] string propertyName = null)
	{
		return base.Set(ref oldValue, newValue, propertyName);
	}

	public virtual bool SetWithDependencies<T>(ref T oldValue, T newValue, string propertyName, params string[] dependentPropertyNames)
	{
		if (Set(ref oldValue, newValue, propertyName))
		{
			foreach (string dependentPropertyName in dependentPropertyNames)
			{
				NotifyOfPropertyChange(dependentPropertyName);
			}

			return true;
		}

		return false;
	}
}