﻿using System;
using System.IO;
using JetBrains.Annotations;
using MimeMapping;
using Newtonsoft.Json;

namespace ECrash.Core.Models.Document;

[JsonConverter(typeof(FilenameJsonConverter))]
public struct Filename : IEquatable<Filename>
{
	private readonly string _filename;

	public Filename([NotNull] string filename)
	{
		_filename = SanitizeFilename(filename ?? throw new ArgumentNullException(nameof(filename)));
	}

	public string Extension => Path.GetExtension(_filename);
	public string MimeType => MimeUtility.GetMimeMapping(_filename);
	public bool Equals(Filename other) => string.Equals(_filename, other._filename, StringComparison.InvariantCultureIgnoreCase);
	public override bool Equals(object obj) => obj is Filename other && Equals(other);
	public override int GetHashCode() => StringComparer.InvariantCultureIgnoreCase.GetHashCode(_filename);
	public static bool operator ==(Filename left, Filename right) => left.Equals(right);
	public static bool operator !=(Filename left, Filename right) => !left.Equals(right);
	public override string ToString() => _filename;
	public static explicit operator string(Filename filename) => filename._filename;
	public static explicit operator Filename(string filename) => new Filename(filename);

	private static string SanitizeFilename(string filename)
	{
		filename = Path.GetFileName(filename);
		return filename;
	}
}