﻿using System;
using System.Threading;
using Caliburn.Micro;
using ECrash.Core.Ioc;

namespace ECrash.Core.Models.ReferenceData;

[Singleton]
public class ReferenceDataCache : IReferenceDataCache
{
	private readonly IEventAggregator _eventAggregator;
	private ReferenceDataList _referenceData;

	public ReferenceDataCache(IEventAggregator eventAggregator)
	{
		_eventAggregator = eventAggregator;
	}

	public ReferenceDataList GetAll()
	{
		var referenceData = Interlocked.CompareExchange(ref _referenceData, null, null);
		if (referenceData == null) throw new InvalidOperationException("ReferenceDataCache has not been initialized");
		return referenceData;
	}

	public ReferenceDataFilter GetFiltered(ReferenceDataEpoch epoch)
	{
		var referenceData = GetAll();
		return new ReferenceDataFilter(referenceData, epoch);
	}

	public void Update(ReferenceDataList referenceData)
	{
		_eventAggregator?.PublishOnUIThreadAsync(new ReferenceDataCacheUpdatingEvent());
		Interlocked.CompareExchange(ref _referenceData, referenceData, _referenceData);
		_eventAggregator?.PublishOnUIThreadAsync(new ReferenceDataCacheUpdatedEvent());
	}
}