﻿using System.Linq;
using Caliburn.Micro;
using Newtonsoft.Json.Linq;

namespace ECrash.Core.Models.ReferenceData;

public static class ReferenceDataExtensions
{
	public static T AddTags<T>(this T referenceData, params string[] tags) where T : ReferenceDataBase
	{
		tags.Apply(tag => referenceData.Tags.Add(tag));
		return referenceData;
	}

	public static T RemoveTags<T>(this T referenceData, params string[] tags) where T : ReferenceDataBase
	{
		tags.Apply(tag => referenceData.Tags.Remove(tag));
		return referenceData;
	}

	public static bool HasTags<T>(this T referenceData, params string[] tags) where T : ReferenceDataBase
	{
		return tags.All(tag => referenceData.Tags.Contains(tag));
	}

	public static T SetField<T>(this T referenceData, string fieldName, object fieldValue) where T : ReferenceDataBase
	{
		referenceData.Fields[fieldName] = JToken.FromObject(fieldValue);
		return referenceData;
	}
}