﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ECrash.Core.Models.ReferenceData;

public class ReferenceDataFilter
{
	private readonly ReferenceDataList _referenceData;
	private readonly ReferenceDataEpoch _epoch;

	public ReferenceDataFilter(ReferenceDataList referenceData, ReferenceDataEpoch epoch)
	{
		_referenceData = referenceData;
		_epoch = epoch;
	}

	public IEnumerable<T> Get<T>() where T : ReferenceDataBase
	{
		return _referenceData.GetDataInEffectAsOf<T>(_epoch);
	}

	public T Get<T>(int id) where T : ReferenceDataBase
	{
		return _referenceData.GetById<T>(id);
	}

	public IEnumerable<T> GetWhere<T>(Func<T, bool> predicate) where T : ReferenceDataBase
	{
		return Get<T>().Where(predicate);
	}

	public T GetFirst<T>(Func<T, bool> predicate) where T : ReferenceDataBase
	{
		return Get<T>().First(predicate);
	}

	public T GetFirst<T>(string code) where T : ReferenceDataBase
	{
		return GetFirst<T>(x => x.Code == code);
	}

	public T GetFirstOrDefault<T>(Func<T, bool> predicate) where T : ReferenceDataBase
	{
		return Get<T>().FirstOrDefault(predicate);
	}

	public T GetFirstOrDefault<T>(string code) where T : ReferenceDataBase
	{
		return GetFirstOrDefault<T>(x => x.Code == code);
	}
	
	public T GetFirstWithTag<T>(string tag) where T : ReferenceDataBase
	{
		return GetFirstOrDefaultWithTag<T>(tag) ?? throw new InvalidOperationException($"No value of type {typeof(T).Name} with tag {tag} was found.");
	}

	public T GetFirstOrDefaultWithTag<T>(string tag) where T : ReferenceDataBase
	{
		string defaultTag = CommonTags.Default(tag);
		T first = null;
		foreach (var value in GetWhere<T>(x => x.Tags.Contains(tag)))
		{
			first ??= value;
			if (value.Tags.Contains(defaultTag)) return value;
		}
		return first;
	}

	public T GetYes<T>() where T : ReferenceDataBase => GetFirstWithTag<T>(CommonTags.YesTag);
	public T GetNo<T>() where T : ReferenceDataBase => GetFirstWithTag<T>(CommonTags.NoTag);
	public T GetUnknown<T>() where T : ReferenceDataBase => GetFirstWithTag<T>(CommonTags.UnknownTag);
	public T GetNotApplicable<T>() where T : ReferenceDataBase => GetFirstWithTag<T>(CommonTags.NotApplicableTag);
}