using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrash.Core.Models.ReferenceData;

public class ReferenceDataListJsonConverter : JsonConverter
{
	public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
	{
		var referenceData = (ReferenceDataList)value;
		var previousTypeNameHandling = serializer.TypeNameHandling;
		var previousTypeNameAssemblyFormatHandling = serializer.TypeNameAssemblyFormatHandling;
		try
		{
			serializer.TypeNameHandling = TypeNameHandling.Auto;
			serializer.TypeNameAssemblyFormatHandling = TypeNameAssemblyFormatHandling.Simple;
			writer.WriteStartObject();
			foreach (var keyValuePair in referenceData.ReferenceData)
			{
				string typeName = keyValuePair.Key;
				int commaIndex = typeName.IndexOf(',');
				if (commaIndex >= 0)
				{
					int startIndex = typeName.LastIndexOf('.', commaIndex) + 1;
					typeName = typeName.Substring(startIndex, commaIndex - startIndex);
				}
				writer.WritePropertyName(typeName);
				writer.WriteStartArray();
				foreach (var referenceDataValue in keyValuePair.Value)
				{
					serializer.Serialize(writer, referenceDataValue);
				}
				writer.WriteEndArray();
			}
			writer.WriteEndObject();
		}
		finally
		{
			serializer.TypeNameHandling = previousTypeNameHandling;
			serializer.TypeNameAssemblyFormatHandling = previousTypeNameAssemblyFormatHandling;
		}
	}

	public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
	{
		var previousTypeNameHandling = serializer.TypeNameHandling;
		var previousTypeNameAssemblyFormatHandling = serializer.TypeNameAssemblyFormatHandling;
		try
		{
			serializer.TypeNameHandling = TypeNameHandling.Auto;
			serializer.TypeNameAssemblyFormatHandling = TypeNameAssemblyFormatHandling.Simple;

			var dictionary = new Dictionary<string, List<ReferenceDataBase>>();
			var obj = JObject.Load(reader);
			foreach (var keyValuePair in obj)
			{
				string typeName = keyValuePair.Key;
				int commaIndex = typeName.IndexOf(',');
				if (commaIndex >= 0)
				{
					int startIndex = typeName.LastIndexOf('.', commaIndex) + 1;
					typeName = typeName.Substring(startIndex, commaIndex - startIndex);
				}
				var type = ReferenceDataMetadata.Types.FirstOrDefault(t => t.Name == typeName) ?? typeof(UnknownReferenceDataType);
				var values = (JArray)keyValuePair.Value;
				var list = values.Select(value => (ReferenceDataBase)value.ToObject(type)).ToList();
				dictionary[typeName] = list;
			}
			return new ReferenceDataList(dictionary);
		}
		finally
		{
			serializer.TypeNameHandling = previousTypeNameHandling;
			serializer.TypeNameAssemblyFormatHandling = previousTypeNameAssemblyFormatHandling;
		}
	}

	public override bool CanConvert(Type objectType)
	{
		return objectType == typeof(ReferenceDataList);
	}
}