﻿using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrash.Core.Models.ReferenceData;

public abstract class ReferenceDataModificationBase
{
	public string Type { get; set; }
	public string Code { get; set; }
	public string Name { get; set; }
	public string Category { get; set; }
	public double Order { get; set; }
	public DateTime? EffectiveDate { get; set; }
	public DateTime? ExpirationDate { get; set; }
	public Version EffectiveVersion { get; set; }
	public Version ExpirationVersion { get; set; }
	public ISet<string> Tags { get; } = new HashSet<string>();
	public IDictionary<string, JToken> Fields { get; } = new Dictionary<string, JToken>();

	public string TagText
	{
		get => string.Join(Environment.NewLine, Tags);
		set
		{
			Tags.Clear();
			if (value == null) return;
			foreach (string tag in value.Split(new[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries).Select(x => x.Trim()))
			{
				if (tag.Length > 0)
				{
					Tags.Add(tag);
				}
			}
		}
	}

	public string FieldText
	{
		get => JsonConvert.SerializeObject(Fields);
		set
		{
			Fields.Clear();
			var fields = JsonConvert.DeserializeObject<Dictionary<string, JToken>>(value);
			foreach (var pair in fields)
			{
				Fields.Add(pair);
			}
		}
	}

	public abstract void Apply(ReferenceDataList referenceData);
}