﻿using System;
using Newtonsoft.Json;

namespace ECrash.Core.Models;

public class TimeJsonConverter : JsonConverter
{
	public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
	{
		if (!(value is Time time))
		{
			writer.WriteNull();
		}
		else
		{
			writer.WriteValue($"{time.Hour:00}:{time.Minute:00}:00");
		}
	}

	public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
	{
		if (!(reader.Value is string s)) return null;
		var parts = s.Split(':');
		if (parts.Length != 3) throw new FormatException("Value must be in the format hh:mm:ss");
		return new Time(int.Parse(parts[0]), int.Parse(parts[1]));
	}

	public override bool CanConvert(Type objectType)
	{
		return objectType == typeof(Time) || objectType == typeof(Time?);
	}
}