﻿using System.Collections.Generic;

namespace ECrash.Core.Models;

public class UtmGeographicPolygon
{
	public int Zone { get; set; }
	public List<GeographicPoint> Points { get; } = new();

	public bool IsPointWithin(double x, double y)
	{
		// Ray-crossing algorithm
		int crossingsCount = 0;

		for (int i = 0; i < Points.Count; i++)
		{
			// Check to see if a ray pointing to the right from "point"
			// crosses the line segment
			var lineEnd1 = Points[i];
			var lineEnd2 = Points[(i + 1) % Points.Count];

			// point.Y is vertically between the line endings' Y coordinate
			if (((lineEnd1.Y <= y) && (lineEnd2.Y > y)) ||
			    ((lineEnd1.Y > y) && (lineEnd2.Y <= y)))
			{
				double slope = (y - lineEnd1.Y) / (lineEnd2.Y - lineEnd1.Y);

				// point.X < intersection
				if (x < lineEnd1.X + slope * (lineEnd2.X - lineEnd1.X))
				{
					crossingsCount++;
				}
			}
		}
		// If there are an odd number of crossings, the point is inside
		// the polygon... else, it is outside
		return (crossingsCount % 2 == 1);
	}
}