﻿using System;
using System.Collections.Specialized;
using System.Linq;
using ECrash.Core.Models.Document;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	public class AttachmentList : DocumentChildCollection<CrashReport, Attachment>
	{
		[JsonIgnore]
		public int TotalLength { get { return Items.Sum(a => a.Length); } }

		protected override void InsertItemBase(int index, Attachment item)
		{
			// Don't allow duplicate attachments
			if (this.FirstOrDefault(a => a == item) is var duplicate && duplicate != null)
			{
				throw new AttachmentDuplicateException(duplicate.AttachmentId, duplicate.Filename, duplicate.Length, duplicate.Hash);
			}

			base.InsertItemBase(index, item);
		}

		protected override void SetItemBase(int index, Attachment item)
		{
			// Don't allow duplicate attachments
			for (int i = 0; i < Count; ++i)
			{
				if (Items[i] == item && i != index)
				{
					throw new AttachmentDuplicateException(Items[i].AttachmentId, Items[i].Filename, Items[i].Length, Items[i].Hash);
				}
			}

			base.SetItemBase(index, item);
		}

		protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
		{
			base.OnCollectionChanged(e);
			NotifyOfPropertyChange("TotalLength");
		}
	}
}