﻿using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.DriverRules
{
	public class ClearDriverLicenseFieldsIfDriverLicenseStatusDoesNotRequireInformation : CrashReportChildBusinessRuleBase<Driver>
	{
		public ClearDriverLicenseFieldsIfDriverLicenseStatusDoesNotRequireInformation(IReferenceDataCache referenceDataCache)
			: base(referenceDataCache, nameof(Driver.DriverLicenseStatus))
		{
		}

		protected override void ApplyRule(Driver model, ReferenceDataFilter referenceData)
		{
			if (model.DriverLicenseStatus?.RequiresLicenseInformation == false)
			{
				model.DriverLicenseState = null;
				model.DriverLicenseStateDescription = null;
				model.DriverLicenseNumber = null;
				model.DriverLicenseClass = referenceData.GetNotApplicable<DriverLicenseClass>();
				model.DriverLicenseType = referenceData.GetNotApplicable<DriverLicenseType>();
				model.CommercialDriverLicenseStatus = referenceData.GetNotApplicable<CommercialDriverLicenseStatus>();
				model.CommercialDriverLicenseStatusDescription = null;

				model.DriverLicenseEndorsements.Clear();
				if (model.DriverLicenseStatus.NotLicensed)
				{
					model.DriverLicenseEndorsements.Add(referenceData.GetNo<DriverLicenseEndorsement>());
				}
				else if (model.DriverLicenseStatus.IsUnknown)
				{
					model.DriverLicenseEndorsements.Add(referenceData.GetUnknown<DriverLicenseEndorsement>());
				}
				model.DriverLicenseEndorsementsDescription = null;

				model.DriverLicenseRestrictions.Clear();
				if (model.DriverLicenseStatus.NotLicensed)
				{
					model.DriverLicenseRestrictions.Add(referenceData.GetNo<DriverLicenseRestriction>());
				}
				else if (model.DriverLicenseStatus.IsUnknown)
				{
					model.DriverLicenseRestrictions.Add(referenceData.GetUnknown<DriverLicenseRestriction>());
				}
				model.DriverLicenseRestrictionsDescription = null;
			}
		}
	}
}