﻿using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.DriverRules
{
	public class SetDriverLicenseTypeBasedOnDriverLicenseClass : CrashReportChildBusinessRuleBase<Driver>
	{
		public SetDriverLicenseTypeBasedOnDriverLicenseClass(IReferenceDataCache referenceDataCache)
			: base(referenceDataCache, nameof(Driver.DriverLicenseClass))
		{
		}

		protected override void ApplyRule(Driver model, ReferenceDataFilter referenceData)
		{
			if (model?.DriverLicenseClass == null) return;

			// If DL Class is None or Not Applicable, then DL Type must be Not Applicable
			if (model.DriverLicenseClass.IsNo || model.DriverLicenseClass.IsNotApplicable)
			{
				model.DriverLicenseType = referenceData.GetNotApplicable<DriverLicenseType>();
				return;
			}

			// If DL Class is always commercial, then DL Type must be Commercial
			if (model.DriverLicenseClass.IsAlwaysCommercial)
			{
				model.DriverLicenseType = referenceData.GetFirst<DriverLicenseType>(x => x.IsCommercial);
				return;
			}

			// If DL CLass is not commercial and DL Type is commercial,
			// then clear DL Type to force the user to choose a valid value
			if (!model.DriverLicenseClass.IsCommercial && model.DriverLicenseType?.IsCommercial == true)
			{
				model.DriverLicenseType = null;
				return;
			}

			// If DL Class is not None or Not Applicable (checked above) and DL Type is Not Applicable,
			// then clear DL Type to force the user to choose a valid value
			if (model.DriverLicenseType?.IsNotApplicable == true)
			{
				model.DriverLicenseType = null;
				// No return here; we might set it to commercial in the next step
			}

			// If DL CLass could be commercial and DL Type is null, then set DL Type to commercial by default
			if (model.DriverLicenseClass.IsCommercial && model.DriverLicenseType == null)
			{
				model.DriverLicenseType = referenceData.GetFirst<DriverLicenseType>(x => x.IsCommercial);
			}
		}
	}
}