﻿using System;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules
{
	public class GuessDistanceFromIntersectionUnitFromMeasurement : CrashReportBusinessRuleBase
	{
		public const decimal AutoUnitThreshold = 5m;

		public GuessDistanceFromIntersectionUnitFromMeasurement(IReferenceDataCache referenceDataCache) : base(referenceDataCache, nameof(CrashReport.DistanceFromIntersection))
		{
		}

		protected override void ApplyRule(CrashReport document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
		{
			var value = document.DistanceFromIntersection;
			if (value == null) return;
			// If the value is less than the threshold or has a decimal component, select Miles, otherwise select Feet
			if (value < AutoUnitThreshold || (value.Value - Math.Truncate(value.Value) != 0))
			{
				document.DistanceFromIntersectionUnit = DistanceUnit.Miles;
			}
			else
			{
				document.DistanceFromIntersectionUnit = DistanceUnit.Feet;
			}
		}
	}
}