﻿using ECrash.Core.Models;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.PersonRules
{
	public class SetAgeFromDateOfBirthAndCrashDate : CrashReportChildBusinessRuleBase<Person>
	{
		public SetAgeFromDateOfBirthAndCrashDate(IReferenceDataCache referenceDataCache) 
			: base(referenceDataCache, nameof(Person.DateOfBirth), nameof(Person.IsDateOfBirthUnknown))
		{
		}

		protected override void OnPropertyChangeCore(CrashReport document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
		{
			base.OnPropertyChangeCore(document, referenceData, change);
			if (change.Source == document && change.IsExplicitlyFor(nameof(CrashReport.CrashDate)))
			{
				foreach (var person in document.EnumeratePersons())
				{
					ApplyRule(person, referenceData);
				}
			}
		}

		protected override void ApplyRule(Person model, ReferenceDataFilter referenceData)
		{
			var crashDate = model.Root.CrashDate;
			var birthDate = model.DateOfBirth;
			if (crashDate != null && birthDate != null)
			{
				model.Age = model.ComputeAgeAsOf(crashDate);
			}
			else if (!model.IsDateOfBirthUnknown)
			{
				model.Age = null;
			}
		}
	}
}