﻿using ECrash.Core.Models;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.Document.BusinessRules;
using ECrash.Core.Models.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.SequenceOfEventsItemRules
{
	public class NonHarmfulEventCannotBeMostHarmfulEvent : DocumentBusinessRuleBase<CrashReport>
	{
		public NonHarmfulEventCannotBeMostHarmfulEvent(IReferenceDataCache referenceDataCache) : base(referenceDataCache)
		{
		}

		protected override void OnPropertyChangeCore(CrashReport document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
		{
			if (change.Source is SequenceOfEventsItem item
			    && (change.IsFor(nameof(SequenceOfEventsItem.Event)) || change.IsFor(nameof(SequenceOfEventsItem.IsMostHarmfulEvent))))
			{
				ApplyRule(item);
			}
		}

		protected override void OnCollectionChangeCore(CrashReport document, ReferenceDataFilter referenceData, DocumentCollectionChangedEventArgs change)
		{
			if (change.Source is SequenceOfEvents sequenceOfEvents)
			{
				ApplyRule(sequenceOfEvents);
			}
			else if (change.Source is VehicleList vehicles)
			{
				foreach (var vehicle in vehicles)
				{
					ApplyRule(vehicle.SequenceOfEvents);
				}
			}
		}

		private static void ApplyRule(SequenceOfEvents sequenceOfEvents1)
		{
			foreach (var item in sequenceOfEvents1)
			{
				ApplyRule(item);
			}
		}

		private static void ApplyRule(SequenceOfEventsItem item)
		{
			if (item.IsMostHarmfulEvent && (item.Event == null || !item.Event.IsHarmful))
			{
				item.IsMostHarmfulEvent = false;
			}
		}
	}
}