using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules
{
	public class SetHighwayBasedOnRoadClassification : CrashReportBusinessRuleBase
	{
		public SetHighwayBasedOnRoadClassification(IReferenceDataCache referenceDataCache) : base(referenceDataCache, nameof(CrashReport.RoadClassification))
		{
		}

		protected override void ApplyRule(CrashReport document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
		{
			var roadClassification = document.RoadClassification;
			if (roadClassification == null || !roadClassification.IsHighway)
			{
				document.Highway = referenceData.GetNotApplicable<Highway>();
			}
			else
			{
				var highway = document.Highway;
				if (highway == null) return;

				if (roadClassification.IsInterstate && !highway.IsInterstate 
				    || roadClassification.IsUSHighway && !highway.IsUSHighway 
				    || roadClassification.IsState && !highway.IsState 
				    || roadClassification.IsHighway && highway.IsNotApplicable)
				{
					document.Highway = null;
				}
			}
		}
	}
}