﻿using System;
using System.Reflection;
using ECrash.Core.Models;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.Document.BusinessRules;
using ECrash.Core.Models.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules
{
	public class SetLastUpdatedWithVersionOnChange : DocumentBusinessRuleBase<CrashReport>
	{
		private static readonly Version Version;

		static SetLastUpdatedWithVersionOnChange()
		{
			Version = Assembly.GetExecutingAssembly().GetName().Version;
		}

		public SetLastUpdatedWithVersionOnChange(IReferenceDataCache referenceDataCache) : base(referenceDataCache)
		{
		}

		protected override void OnPropertyChangeCore(CrashReport document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
		{
			if (change.IsExplicitlyFor(nameof(CrashReport.LastUpdatedWithVersion))) return;
			document.LastUpdatedWithVersion = Version;
		}

		protected override void OnCollectionChangeCore(CrashReport document, ReferenceDataFilter referenceData, DocumentCollectionChangedEventArgs change)
		{
			document.LastUpdatedWithVersion = Version;
		}
	}
}