﻿using System;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules
{
	public class SetWorkZoneFieldsBasedOnWorkZoneRelation : CrashReportBusinessRuleBase
	{
		public SetWorkZoneFieldsBasedOnWorkZoneRelation(IReferenceDataCache referenceDataCache) 
			: base(referenceDataCache, nameof(CrashReport.WorkZoneRelation))
		{
		}

		protected override void ApplyRule(CrashReport document, ReferenceDataFilter referenceData, DocumentPropertyChangedEventArgs change)
		{
			if (document.WorkZoneRelation == null || document.WorkZoneRelation.IsYes)
			{
				ClearNotApplicableOrUnknown(() => document.WorkZoneLocation, v => document.WorkZoneLocation = v);
				ClearNotApplicableOrUnknown(() => document.WorkZoneType, v => document.WorkZoneType = v);
				ClearNotApplicableOrUnknown(() => document.WorkZoneCircumstance, v => document.WorkZoneCircumstance = v);
				ClearNotApplicableOrUnknown(() => document.WorkerPresence, v => document.WorkerPresence = v);
				ClearNotApplicableOrUnknown(() => document.LawEnforcementPresence, v => document.LawEnforcementPresence = v);
			}
			else if (document.WorkZoneRelation.IsNo)
			{
				document.WorkZoneLocation = referenceData.GetFirstOrDefault<WorkZoneLocation>(v => v.IsNotApplicable);
				document.WorkZoneType = referenceData.GetFirstOrDefault<WorkZoneType>(v => v.IsNotApplicable);
				document.WorkZoneCircumstance = referenceData.GetFirstOrDefault<WorkZoneCircumstance>(v => v.IsNotApplicable);
				document.WorkerPresence = referenceData.GetFirstOrDefault<WorkerPresence>(v => v.IsNotApplicable);
				document.LawEnforcementPresence = referenceData.GetFirstOrDefault<LawEnforcementPresence>(v => v.IsNotApplicable);
			}
			else if (document.WorkZoneRelation.IsUnknown)
			{
				document.WorkZoneLocation = referenceData.GetFirstOrDefault<WorkZoneLocation>(v => v.IsUnknown);
				document.WorkZoneType = referenceData.GetFirstOrDefault<WorkZoneType>(v => v.IsUnknown);
				document.WorkZoneCircumstance = referenceData.GetFirstOrDefault<WorkZoneCircumstance>(v => v.IsUnknown);
				document.WorkerPresence = referenceData.GetFirstOrDefault<WorkerPresence>(v => v.IsUnknown);
				document.LawEnforcementPresence = referenceData.GetFirstOrDefault<LawEnforcementPresence>(v => v.IsUnknown);
			}
		}

		private void ClearNotApplicableOrUnknown<T>(Func<T> getter, Action<T> setter) where T : ReferenceDataBase
		{
			var value = getter();
			if (value != null && (value.IsNotApplicable || value.IsUnknown))
			{
				setter(null);
			}
		}
	}
}