﻿using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.TrainPassengerRules
{
	public class SetEmsResponseAndMedicalFacilityFieldsBasedOnMedicalTransportationType : CrashReportChildBusinessRuleBase<TrainPassenger>
	{
		public SetEmsResponseAndMedicalFacilityFieldsBasedOnMedicalTransportationType(IReferenceDataCache referenceDataCache) : base(referenceDataCache, nameof(Person.MedicalTransportationType))
		{
		}

		protected override void ApplyRule(TrainPassenger model, ReferenceDataFilter referenceData)
		{
			if (model.MedicalTransportationType == null) return;

			if (model.MedicalTransportationType.IsUnknown)
			{
				model.EmsResponseAgency = referenceData.GetUnknown<EmergencyMedicalService>();
				model.EmsResponseAgencyDescription = null;
				model.IsEmsResponseRunNumberUnknown = true;
				model.EmsResponseRunNumber = null;
				model.MedicalFacilityReceivingPatient = referenceData.GetUnknown<MedicalFacility>();
				model.MedicalFacilityReceivingPatientDescription = null;
				return;
			}

			if (model.MedicalTransportationType.IsNo)
			{
				model.MedicalFacilityReceivingPatient = referenceData.GetNotApplicable<MedicalFacility>();
				model.MedicalFacilityReceivingPatientDescription = null;
			}
			else if (model.MedicalFacilityReceivingPatient?.IsNotApplicable == true)
			{
				model.MedicalFacilityReceivingPatient = null;
			}

			if (!model.MedicalTransportationType.IsEmergencyMedicalService)
			{
				model.EmsResponseAgency = referenceData.GetNotApplicable<EmergencyMedicalService>();
				model.EmsResponseAgencyDescription = null;
				model.IsEmsResponseRunNumberUnknown = false;
				model.EmsResponseRunNumber = null;
				
			}
			else if (model.EmsResponseAgency?.IsNotApplicable == true)
			{
				model.EmsResponseAgency = null;
			}
		}
	}
}