﻿using System;
using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetAutomationSystemLevelEngagedBasedOnVehicleTypeAndAutomationSystemLevelPresent : CrashReportChildBusinessRuleBase<Vehicle>
	{
		public SetAutomationSystemLevelEngagedBasedOnVehicleTypeAndAutomationSystemLevelPresent(IReferenceDataCache referenceDataCache)
			: base(referenceDataCache, nameof(Vehicle.AutomationSystemLevelPresent), nameof(Vehicle.Type))
		{
		}

		protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
		{
			try
			{
				if (model.Type?.IsParked == true)
				{
					// ASLE = No if parked
					model.AutomationSystemLevelEngaged = referenceData.GetNo<VehicleAutomationSystemLevel>();
				}
				else if (model.AutomationSystemLevelPresent == null || (model.AutomationSystemLevelPresent.IsYes && !model.AutomationSystemLevelPresent.IsUnknown))
				{
					// Clear ASLE if it is No or Unknown when ASLP is null or Yes and not Unknown
					ClearNoOrUnknown(() => model.AutomationSystemLevelEngaged, v => model.AutomationSystemLevelEngaged = v);
				}
				else if (model.AutomationSystemLevelPresent.IsNo)
				{
					// ASLE = No if ASLP = No
					model.AutomationSystemLevelEngaged = referenceData.GetFirstOrDefault<VehicleAutomationSystemLevel>(v => v.IsNo);
				}
				else if (model.AutomationSystemLevelPresent.IsUnknown)
				{
					// ASLE = Unknown if ASLP = Unknown
					model.AutomationSystemLevelEngaged = referenceData.GetFirstOrDefault<VehicleAutomationSystemLevel>(v => v.IsUnknown && v.IsYes == model.AutomationSystemLevelPresent.IsYes);
				}
			}
			finally
			{
				if (model.AutomationSystemLevelEngaged?.CanBeEngaged(model.AutomationSystemLevelPresent) == false)
				{
					// If ASLE is not valid for ASLP, clear ASLE
					model.AutomationSystemLevelEngaged = null;
				}
			}
		}

		private void ClearNoOrUnknown<T>(Func<T> getter, Action<T> setter) where T : ReferenceDataBase
		{
			var value = getter();
			if (value != null && (value.IsNo || value.IsUnknown))
			{
				setter(null);
			}
		}
	}
}