using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetCommercialVehicleFieldsBasedOnVehicleConfiguration : CrashReportChildBusinessRuleBase<Vehicle>
	{
		public SetCommercialVehicleFieldsBasedOnVehicleConfiguration(IReferenceDataCache referenceDataCache) : base(referenceDataCache, nameof(Vehicle.VehicleConfiguration))
		{
		}

		protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
		{
			if (model.VehicleConfiguration == null) return;
			if (model.VehicleConfiguration.IsNo)
			{
				model.CargoBodyType = referenceData.GetNotApplicable<CargoBodyType>();
				model.IsCommodityHauledUnknown = false;
				model.CommodityHauled = null;
				model.WeightRating = referenceData.GetFirst<WeightRating>(x => x.IsLightWeight);
				model.VehicleSpecialSizings.Clear();
				model.VehicleSpecialSizings.Add(referenceData.GetNo<VehicleSpecialSizing>());
				model.VehicleLoadPermit = referenceData.GetNotApplicable<VehicleLoadPermit>();
				model.NumberOfAxles = null;
				model.IsNumberOfAxlesUnknown = false;
			}
			else if (model.VehicleConfiguration.IsUnknown)
			{
				model.CargoBodyType = referenceData.GetUnknown<CargoBodyType>();
				model.IsCommodityHauledUnknown = true;
				model.CommodityHauled = null;
				model.WeightRating = referenceData.GetUnknown<WeightRating>();
				model.VehicleSpecialSizings.Clear();
				model.VehicleSpecialSizings.Add(referenceData.GetUnknown<VehicleSpecialSizing>());
				model.VehicleLoadPermit = referenceData.GetUnknown<VehicleLoadPermit>();
				model.IsNumberOfAxlesUnknown = true;
				model.NumberOfAxles = null;
			}
			else
			{
				if (model.CargoBodyType != null && (model.CargoBodyType.IsNotApplicable || model.CargoBodyType.IsUnknown))
				{
					model.CargoBodyType = null;
				}
				model.IsCommodityHauledUnknown = false;
				if (model.WeightRating != null && (model.WeightRating.IsNotApplicable || model.WeightRating.IsUnknown))
				{
					model.WeightRating = null;
				}
				if (model.VehicleConfiguration.IsWeight10kOrBelow)
				{
					model.WeightRating = referenceData.GetFirst<WeightRating>(x => x.IsLightWeight);
				}
				if (model.VehicleSpecialSizings.Count == 1 && (model.VehicleSpecialSizings[0].IsNo || model.VehicleSpecialSizings[0].IsUnknown))
				{
					model.VehicleSpecialSizings.Clear();
				}
				if (model.VehicleLoadPermit != null && (model.VehicleLoadPermit.IsNotApplicable || model.VehicleLoadPermit.IsUnknown))
				{
					model.VehicleLoadPermit = null;
				}
				model.IsNumberOfAxlesUnknown = false;
			}
		}
	}
}