﻿using System;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetOccupantEjectionToNotApplicableIfVehicleBodyTypeChangesToMotorcycle : CrashReportChildBusinessRuleBase<Vehicle>
	{
		public SetOccupantEjectionToNotApplicableIfVehicleBodyTypeChangesToMotorcycle(IReferenceDataCache referenceDataCache, params string[] properties) 
			: base(referenceDataCache, nameof(Vehicle.BodyType))
		{
		}

		protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
		{
			if (model.BodyType?.IsNoEjection == true)
			{
				if (model.Driver != null)
				{
					model.Driver.Ejection = referenceData.GetNotApplicable<Ejection>();
				}

				if (model.Passengers != null && model.Passengers.Count != 0)
				{
					foreach (var passenger in model.Passengers)
					{
						passenger.Ejection = referenceData.GetNotApplicable<Ejection>();
					}
				}
			}
		}

		protected override void OnNodeAttached(CrashReport document, IDocumentNode<CrashReport> node, ReferenceDataFilter referenceData)
		{
			base.OnNodeAttached(document, node, referenceData);
			if (node is Occupant occupant && occupant.GetParentVehicle().BodyType?.IsNoEjection == true)
			{
				occupant.Ejection = referenceData.GetNotApplicable<Ejection>();
			}
		}
	}
}