﻿using ECrash.Core.Models;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetOwnerAddressIfSameAsDriver : CrashReportChildBusinessRuleBase<Vehicle>
	{
		private static readonly string[] DriverProperties =
		{
			nameof(Driver.AddressStreet),
			nameof(Driver.AddressCity),
			nameof(Driver.AddressState),
			nameof(Driver.AddressStateDescription),
			nameof(Driver.AddressPostalCode)
		};

		public SetOwnerAddressIfSameAsDriver(IReferenceDataCache referenceDataCache) 
			: base(referenceDataCache, nameof(Vehicle.IsOwnerAddressSameAsDriver), nameof(Vehicle.Driver))
		{
		}

		protected override bool ShouldApplyRule(DocumentPropertyChangedEventArgs change, out Vehicle child)
		{
			if (base.ShouldApplyRule(change, out child)) return true;
			if (change.Source is Driver driver && change.IsForAny(DriverProperties))
			{
				child = (Vehicle)driver.Parent;
				return true;
			}
			return false;
		}

		protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
		{
			if (model.IsOwnerAddressSameAsDriver)
			{
				model.OwnerAddressStreet = model.Driver?.AddressStreet;
				model.OwnerAddressCity = model.Driver?.AddressCity;
				model.OwnerAddressState = model.Driver?.AddressState;
				model.OwnerAddressStateDescription = model.Driver?.AddressStateDescription;
				model.OwnerAddressPostalCode = model.Driver?.AddressPostalCode;
			}
		}
	}
}