﻿using ECrash.Core.Models;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetOwnerNameIfSameAsDriver : CrashReportChildBusinessRuleBase<Vehicle>
	{
		private static readonly string[] DriverProperties =
		{
			nameof(Driver.FirstName),
			nameof(Driver.MiddleName),
			nameof(Driver.LastName),
			nameof(Driver.NameSuffix),
		};

		public SetOwnerNameIfSameAsDriver(IReferenceDataCache referenceDataCache) 
			: base(referenceDataCache, nameof(Vehicle.IsOwnerNameSameAsDriver), nameof(Vehicle.Driver))
		{
		}

		protected override bool ShouldApplyRule(DocumentPropertyChangedEventArgs change, out Vehicle child)
		{
			if (base.ShouldApplyRule(change, out child)) return true;
			if (change.Source is Driver driver && change.IsForAny(DriverProperties))
			{
				child = (Vehicle)driver.Parent;
				return true;
			}
			return false;
		}

		protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
		{
			if (model.IsOwnerNameSameAsDriver)
			{
				model.OwnerName = model.Driver?.FullName;
			}
		}
	}
}