﻿using System.Linq;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetTrafficSignalStatusBasedOnTrafficControlDevicesPresent : CrashReportValueCollectionBusinessRuleBase<Vehicle, TrafficControlDeviceTypeList, TrafficControlDeviceType>
	{
		public SetTrafficSignalStatusBasedOnTrafficControlDevicesPresent(IReferenceDataCache referenceDataCache) : base(referenceDataCache)
		{
		}

		protected override void OnInitializeCore(CrashReport document, ReferenceDataFilter referenceData)
		{
			base.OnInitializeCore(document, referenceData);
			foreach (var vehicle in document.Vehicles)
			{
				ApplyRule(vehicle, vehicle.TrafficControlDevicesPresent, referenceData);
			}
		}

		protected override void OnNodeAttached(CrashReport document, IDocumentNode<CrashReport> node, ReferenceDataFilter referenceData)
		{
			base.OnNodeAttached(document, node, referenceData);
			if (node is Vehicle vehicle)
			{
				ApplyRule(vehicle, vehicle.TrafficControlDevicesPresent, referenceData);
			}
		}

		protected override bool ShouldApplyRule(Vehicle model, TrafficControlDeviceTypeList collection)
		{
			// Only run on the Present collection, not the InoperativeOrMissing collection
			return base.ShouldApplyRule(model, collection) && ReferenceEquals(collection, model.TrafficControlDevicesPresent);
		}

		protected override void ApplyRule(Vehicle model, TrafficControlDeviceTypeList collection, ReferenceDataFilter referenceData)
		{
			bool requiresTrafficSignalStatus = collection.Any(x => x.RequiresTrafficSignalStatus);
			bool trafficControlDevicePresentUnknown = collection.Any(x => x.IsUnknown);
			bool isNotApplicable = model.TrafficSignalStatus?.IsNotApplicable == true;
			if (trafficControlDevicePresentUnknown)
			{
				model.TrafficSignalStatus = referenceData.GetUnknown<TrafficSignalStatus>();
			}
			else if (!requiresTrafficSignalStatus)
			{
				model.TrafficSignalStatus = referenceData.GetNotApplicable<TrafficSignalStatus>();
			}
			else if (requiresTrafficSignalStatus && isNotApplicable)
			{
				model.TrafficSignalStatus = null;
			}
		}
	}
}