﻿using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules;

public class SetTrafficwayFieldsIfNotOnTrafficway : CrashReportChildBusinessRuleBase<Vehicle>
{
	public SetTrafficwayFieldsIfNotOnTrafficway(IReferenceDataCache referenceDataCache) 
		: base(referenceDataCache, nameof(Vehicle.TravelDirection))
	{
	}

	protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
	{
		if (model == null) return;
		
		if (model.TravelDirection == null || !model.TravelDirection.IsNotOnTrafficway)
		{
			if (model.TrafficwayTravelDirection?.IsNotOnTrafficway == true) model.TrafficwayTravelDirection = null;
			if (model.RoadwayAlignment?.IsNotOnTrafficway == true) model.RoadwayAlignment = null;
			if (model.RoadwayGrade?.IsNotOnTrafficway == true) model.RoadwayGrade = null;
		}
		else
		{
			model.TrafficwayTravelDirection = referenceData.GetFirstWithTag<TrafficwayTravelDirection>(TrafficwayTravelDirection.NotOnTrafficwayTag);
			model.TrafficwayDivision = referenceData.Get<TrafficwayDivision>(1);
			model.TrafficwayBarrierType = referenceData.GetNo<TrafficwayBarrierType>();
			model.TotalThroughLanes = 0;
			model.TotalAuxiliaryLanes = 0;
			model.TrafficwayHovLanePresence = referenceData.GetNo<TrafficwayHovLanePresence>();
			model.TrafficwayHovLaneRelation = referenceData.GetNo<TrafficwayHovLaneRelation>();
			model.RoadwayAlignment = referenceData.GetFirstWithTag<RoadwayAlignment>(RoadwayAlignment.NotOnTrafficwayTag);
			model.RoadwayGrade = referenceData.GetFirstWithTag<RoadwayGrade>(RoadwayGrade.NotOnTrafficwayTag);
			model.SpeedLimit = null;
			model.IsSpeedLimitUnknown = false;
			model.IsSpeedLimitNotApplicable = true;
			model.TrafficControlDevicesPresent.Clear();
			model.TrafficControlDevicesPresent.Add(referenceData.GetNo<TrafficControlDeviceType>());
			model.TrafficSignalStatus = referenceData.GetNotApplicable<TrafficSignalStatus>();
			model.TrafficControlDevicesInoperativeOrMissing.Clear();
			model.TrafficControlDevicesInoperativeOrMissing.Add(referenceData.GetNo<TrafficControlDeviceType>());
		}
	}
}