﻿using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;
using ECrashLA.Core.Models.CrashReports.Validation.Rules.Vehicles;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetVehicleAndDriverFieldsBasedOnHitAndRun : CrashReportChildBusinessRuleBase<Vehicle>
	{
		public SetVehicleAndDriverFieldsBasedOnHitAndRun(IReferenceDataCache referenceDataCache) : base(referenceDataCache, nameof(Vehicle.HitAndRun))
		{
		}

		protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
		{
			var hitAndRun = model.HitAndRun;
			if (hitAndRun == null) return;
            if (hitAndRun.VehicleLeftScene)
			{
				if (string.IsNullOrWhiteSpace(model.Vin)) model.IsVinUnknown = true;
				if (model.BodyType == null) model.BodyType = referenceData.GetUnknown<VehicleBodyType>();
				if (model.ModelYear == null) model.IsModelYearUnknown = true;
				if (model.Make == null) model.Make = referenceData.GetUnknown<VehicleMake>();
				if (model.Model == null) model.Model = referenceData.GetFirstOrDefault<VehicleModel>(x => x.IsUnknown && x.MakeCode == model.Make?.Code);
				if (model.Color == null) model.Color = referenceData.GetUnknown<VehicleColor>();
				if (!model.IsTagMissing)
				{
					if (model.TagYear == null) model.IsTagYearUnknown = true;
					if (model.TagState == null) model.IsTagStateUnknown = true;
					if (string.IsNullOrWhiteSpace(model.TagNumber)) model.IsTagNumberUnknown = true;
				}
				if (string.IsNullOrWhiteSpace(model.OwnerName)) model.IsOwnerNameUnknown = true;
				if (string.IsNullOrWhiteSpace(model.OwnerAddressStreet) && 
				    string.IsNullOrWhiteSpace(model.OwnerAddressCity) && 
				    model.OwnerAddressState == null && 
				    string.IsNullOrWhiteSpace(model.OwnerAddressStateDescription) && 
				    string.IsNullOrWhiteSpace(model.OwnerAddressPostalCode))
				{
					model.IsOwnerAddressUnknown = true;
				}
				if (!model.IsUninsured)
				{
					if (string.IsNullOrWhiteSpace(model.LiabilityInsuranceCompany)) model.IsLiabilityInsuranceCompanyUnknown = true;
					if (string.IsNullOrWhiteSpace(model.LiabilityInsuranceCompanyPhoneNumber)) model.IsLiabilityInsuranceCompanyPhoneNumberUnknown = true;
					if (string.IsNullOrWhiteSpace(model.LiabilityInsuranceNaicNumber)) model.IsLiabilityInsuranceNaicNumberUnknown = true;
					if (string.IsNullOrWhiteSpace(model.LiabilityInsurancePolicyNumber)) model.IsLiabilityInsurancePolicyNumberUnknown = true;
					if (model.LiabilityInsurancePolicyExpirationDate == null) model.IsLiabilityInsurancePolicyExpirationDateUnknown = true;
				}
				if (model.DamageExtent == null) model.DamageExtent = referenceData.GetUnknown<VehicleDamageExtent>();
				if (model.InitialPointOfContact == null) model.InitialPointOfContact = referenceData.GetFirstOrDefault<VehicleArea>(area => area.IsVehicleNotAtScene);
				if (model.DamagedAreas.Count == 0) model.DamagedAreas.Add(referenceData.GetFirstOrDefault<VehicleArea>(x => x.IsVehicleNotAtScene));
				if (model.VehicleUsage == null) model.VehicleUsage = referenceData.GetUnknown<VehicleUsage>();
				if (model.EmergencyVehicleUsage == null) model.EmergencyVehicleUsage = referenceData.GetUnknown<EmergencyVehicleUsage>();
				if (model.Maneuver == null) model.Maneuver = referenceData.GetUnknown<VehicleManeuver>();
				if (model.ManeuverReason == null) model.ManeuverReason = referenceData.GetUnknown<VehicleManeuverReason>();
				if (model.Lighting == null) model.Lighting = referenceData.GetUnknown<VehicleLighting>();
				if (model.TravelDirection == null) model.TravelDirection = referenceData.GetUnknown<VehicleTravelDirection>();
				if (model.ContributingCircumstance == null) model.ContributingCircumstance = referenceData.GetUnknown<VehicleContributingCircumstance>();
				if (model.DistanceTraveledAfterImpact == null) model.IsDistanceTraveledAfterImpactUnknown = true;
				if (model.SkidmarkLengthFrontLeft == null
				    && model.SkidmarkLengthFrontRight == null
				    && model.SkidmarkLengthRearLeft == null
				    && model.SkidmarkLengthRearRight == null)
					model.IsSkidmarkLengthNotApplicable = true;

				if (model.TowStatus == null) model.TowStatus = referenceData.GetNo<TowStatus>();
				if (model.AutomationSystemLevelPresent == null) model.AutomationSystemLevelPresent = referenceData.GetUnknown<VehicleAutomationSystemLevel>();
				if (model.AutomationSystemLevelEngaged == null) model.AutomationSystemLevelEngaged = referenceData.GetUnknown<VehicleAutomationSystemLevel>();
				if (model.VehicleConfiguration == null) model.VehicleConfiguration = referenceData.GetUnknown<VehicleConfiguration>();
				if (model.HazardousMaterialsPlacardStatus == null) model.HazardousMaterialsPlacardStatus = referenceData.GetUnknown<HazardousMaterialsPlacardStatus>();
			}
			else
			{
				model.IsVinUnknown = false;
				if (model.BodyType?.IsUnknown == true) model.BodyType = null;
				model.IsModelYearUnknown = false;
				if (model.Make?.IsUnknown == true) model.Make = null;
				if (model.Model?.IsUnknown == true) model.Model = null;
				if (model.Color?.IsUnknown == true) model.Color = null;
				model.IsTagYearUnknown = false;
				model.IsTagStateUnknown = false;
				model.IsTagNumberUnknown = false;
				model.IsOwnerNameUnknown = false;
				model.IsOwnerAddressUnknown = false;
				model.IsLiabilityInsuranceCompanyUnknown = false;
				model.IsLiabilityInsuranceCompanyPhoneNumberUnknown = false;
				model.IsLiabilityInsuranceNaicNumberUnknown = false;
				model.IsLiabilityInsurancePolicyNumberUnknown = false;
				model.IsLiabilityInsurancePolicyExpirationDateUnknown = false;
				if (model.DamageExtent?.IsUnknown == true) model.DamageExtent = null;
				if (model.InitialPointOfContact?.IsVehicleNotAtScene == true) model.InitialPointOfContact = null;
				model.DamagedAreas.Remove(referenceData.GetFirstOrDefault<VehicleArea>(x => x.IsVehicleNotAtScene));
				if (model.VehicleUsage?.IsUnknown == true) model.VehicleUsage = null;
				if (model.EmergencyVehicleUsage?.IsUnknown == true) model.EmergencyVehicleUsage = null;
				if (model.Maneuver?.IsUnknown == true) model.Maneuver = null;
				if (model.ManeuverReason?.IsUnknown == true) model.ManeuverReason = null;
				if (model.Lighting?.IsUnknown == true) model.Lighting = null;
				if (model.TravelDirection?.IsUnknown == true) model.TravelDirection = null;
				if (model.ContributingCircumstance?.IsUnknown == true) model.ContributingCircumstance = null;
				model.IsDistanceTraveledAfterImpactUnknown = false;
				model.IsSkidmarkLengthNotApplicable = false;
				model.IsSkidmarkLengthUnknown = false;
				if (model.TowStatus?.IsNo == true) model.TowStatus = null;
				if (model.AutomationSystemLevelPresent?.IsUnknown == true) model.AutomationSystemLevelPresent = null;
				if (model.AutomationSystemLevelEngaged?.IsUnknown == true) model.AutomationSystemLevelEngaged = null;
				if (model.VehicleConfiguration?.IsUnknown == true) model.VehicleConfiguration = null;
				if (model.HazardousMaterialsPlacardStatus?.IsUnknown == true) model.HazardousMaterialsPlacardStatus = null;
			}
            if (hitAndRun.DriverLeftScene)
			{
				var driver = model.Driver = model.Driver ?? new Driver(); 
				if (string.IsNullOrWhiteSpace(driver.FirstName) &&
				    string.IsNullOrWhiteSpace(driver.MiddleName) &&
				    string.IsNullOrWhiteSpace(driver.LastName) &&
				    string.IsNullOrWhiteSpace(driver.NameSuffix))
				{
					driver.IsNameUnknown = true;
				}
				if (string.IsNullOrWhiteSpace(driver.AddressStreet) &&
				    string.IsNullOrWhiteSpace(driver.AddressCity) &&
				    driver.AddressState == null &&
				    string.IsNullOrWhiteSpace(driver.AddressStateDescription) &&
				    string.IsNullOrWhiteSpace(driver.AddressPostalCode))
				{
					driver.IsAddressUnknown = true;
				}
				if (string.IsNullOrWhiteSpace(driver.PhoneNumber)) driver.IsPhoneNumberUnknown = true;
				if (driver.DateOfBirth == null) driver.IsDateOfBirthUnknown = true;
				if (driver.IsDateOfBirthUnknown && driver.Age == null) driver.IsAgeUnknown = true;
				if (driver.Sex == null) driver.Sex = referenceData.GetUnknown<Sex>();
				if (driver.Race == null) driver.Race = referenceData.GetUnknown<Race>();
				if (driver.Ethnicity == null) driver.Ethnicity = referenceData.GetUnknown<Ethnicity>();
				if (driver.IncidentResponder == null) driver.IncidentResponder = referenceData.GetUnknown<IncidentResponder>();
				if (driver.InjuryStatus == null) driver.InjuryStatus = referenceData.GetFirstOrDefault<InjuryStatus>(x => !x.IsInjury);
				if (driver.MedicalTransportationType == null) driver.MedicalTransportationType = referenceData.GetUnknown<MedicalTransportationType>();
				if (driver.EmsResponseAgency == null) driver.EmsResponseAgency = referenceData.GetUnknown<EmergencyMedicalService>();
				if (string.IsNullOrWhiteSpace(driver.EmsResponseRunNumber)) driver.IsEmsResponseRunNumberUnknown = true;
				if (driver.MedicalFacilityReceivingPatient == null) driver.MedicalFacilityReceivingPatient = referenceData.GetUnknown<MedicalFacility>();
				if (string.IsNullOrWhiteSpace(driver.MedicalUniqueIdentifier)) driver.IsMedicalUniqueIdentifierUnknown = true;
				if (driver.DriverLicenseStatus == null) driver.DriverLicenseStatus = referenceData.GetUnknown<DriverLicenseStatus>();
				if (driver.RestraintSystemType == null) driver.RestraintSystemType = referenceData.GetFirstOrDefault<RestraintSystemType>(x => x.IsUnknown && !x.IsYes);
				if (driver.ImproperRestraintSystemUsage == null) driver.ImproperRestraintSystemUsage = referenceData.GetUnknown<ImproperRestraintSystemUsage>();
				if (driver.Ejection == null) driver.Ejection = referenceData.GetUnknown<Ejection>();
				if (driver.Extrication == null) driver.Extrication = referenceData.GetNo<Extrication>();
				if (driver.AirBagDeployments.Count == 0 && hitAndRun.VehicleLeftScene) driver.AirBagDeployments.Add(referenceData.GetUnknown<AirBagDeployment>());
				if (driver.ConditionInfo.Conditions.Count == 0) driver.ConditionInfo.Conditions.Add(referenceData.GetUnknown<Condition>());
				if (driver.ConditionInfo.DistractionAction == null) driver.ConditionInfo.DistractionAction = referenceData.GetUnknown<DistractionAction>();
				if (driver.ConditionInfo.DistractionSource == null) driver.ConditionInfo.DistractionSource = referenceData.GetUnknown<DistractionSource>();
				if (driver.ConditionInfo.AlcoholUseSuspicion == null) driver.ConditionInfo.AlcoholUseSuspicion = referenceData.GetUnknown<AlcoholUseSuspicion>();
				if (driver.ConditionInfo.DrugUseSuspicion == null) driver.ConditionInfo.DrugUseSuspicion = referenceData.GetUnknown<DrugUseSuspicion>();
				if (driver.SpeedingRelation == null) driver.SpeedingRelation = referenceData.GetUnknown<SpeedingRelation>();
				if (driver.AvoidanceManeuver == null) driver.AvoidanceManeuver = referenceData.GetUnknown<DriverAvoidanceManeuver>();
				if (driver.PreCollisionStability == null) driver.PreCollisionStability = referenceData.GetUnknown<PreCollisionStability>();
				if (driver.VisionObscurement == null) driver.VisionObscurement = referenceData.GetUnknown<VisionObscurement>();
				if (driver.DriverActions.Count == 0) driver.DriverActions.Add(referenceData.GetUnknown<DriverAction>());
			}
			else if (model.Driver != null)
			{
				var driver = model.Driver;
				driver.IsNameUnknown = false;
				driver.IsAddressUnknown = false;
				driver.IsPhoneNumberUnknown = false;
				driver.IsDateOfBirthUnknown = false;
				driver.IsAgeUnknown = false;
				if (driver.Sex?.IsUnknown == true) driver.Sex = null;
				if (driver.Race?.IsUnknown == true) driver.Race = null;
				if (driver.Ethnicity?.IsUnknown == true) driver.Ethnicity = null;
				if (driver.IncidentResponder?.IsUnknown == true) driver.IncidentResponder = null;
				if (driver.InjuryStatus?.IsInjury == false) driver.InjuryStatus = null;
				if (driver.MedicalTransportationType?.IsUnknown == true) driver.MedicalTransportationType = null;
				if (driver.EmsResponseAgency?.IsUnknown == true) driver.EmsResponseAgency = null;
				driver.IsEmsResponseRunNumberUnknown = false;
				if (driver.MedicalFacilityReceivingPatient?.IsUnknown == true) driver.MedicalFacilityReceivingPatient = null;
				driver.IsMedicalUniqueIdentifierUnknown = false;
				if (driver.DriverLicenseStatus?.IsUnknown == true) driver.DriverLicenseStatus = null;
				if (driver.RestraintSystemType?.IsUnknown == true) driver.RestraintSystemType = null;
				if (driver.ImproperRestraintSystemUsage?.IsUnknown == true) driver.ImproperRestraintSystemUsage = null;
				if (driver.Ejection?.IsUnknown == true) driver.Ejection = null;
				if (driver.Extrication?.IsNo == true) driver.Extrication = null;
				driver.AirBagDeployments.Remove(referenceData.GetUnknown<AirBagDeployment>());
				driver.ConditionInfo.Conditions.Remove(referenceData.GetUnknown<Condition>());
				if (driver.ConditionInfo.DistractionAction?.IsUnknown == true) driver.ConditionInfo.DistractionAction = null;
				if (driver.ConditionInfo.DistractionSource?.IsUnknown == true) driver.ConditionInfo.DistractionSource = null;
				if (driver.ConditionInfo.AlcoholUseSuspicion?.IsUnknown == true) driver.ConditionInfo.AlcoholUseSuspicion = null;
				if (driver.ConditionInfo.DrugUseSuspicion?.IsUnknown == true) driver.ConditionInfo.DrugUseSuspicion = null;
				if (driver.SpeedingRelation?.IsUnknown == true) driver.SpeedingRelation = null;
				if (driver.AvoidanceManeuver?.IsUnknown == true) driver.AvoidanceManeuver = null;
				if (driver.PreCollisionStability?.IsUnknown == true) driver.PreCollisionStability = null;
				if (driver.VisionObscurement?.IsUnknown == true) driver.VisionObscurement = null;
				driver.DriverActions.Remove(referenceData.GetUnknown<DriverAction>());
			}
		}
	}
}