﻿using System;
using System.Linq;
using ECrash.Core.Models.ReferenceData;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports.BusinessRules.VehicleRules
{
	public class SetVehicleDamageFieldsBasedOnDamageExtent : CrashReportChildBusinessRuleBase<Vehicle>
	{
		public SetVehicleDamageFieldsBasedOnDamageExtent(IReferenceDataCache referenceDataCache)
			: base(referenceDataCache, nameof(Vehicle.DamageExtent))
		{
		}

		protected override void ApplyRule(Vehicle model, ReferenceDataFilter referenceData)
		{
			if (model.DamageExtent == null || model.DamageExtent.IsYes)
			{
				if (model.InitialPointOfContact != null && (model.InitialPointOfContact.IsNoDamage || model.InitialPointOfContact.IsVehicleNotAtScene))
				{
					model.InitialPointOfContact = null;
				}

				if (model.DamagedAreas.Any(a => a.IsNoDamage || a.IsVehicleNotAtScene))
				{
					model.DamagedAreas.Clear();
				}
			}
			else if (model.DamageExtent.IsNo)
			{
				model.DamagedAreas.Clear();
				model.DamagedAreas.Add(referenceData.GetFirstOrDefault<VehicleArea>(area => area.IsNoDamage));
			}
			else if (model.DamageExtent.IsUnknown)
			{
				model.InitialPointOfContact = referenceData.GetFirstOrDefault<VehicleArea>(area => area.IsVehicleNotAtScene);
				model.DamagedAreas.Clear();
				model.DamagedAreas.Add(referenceData.GetFirstOrDefault<VehicleArea>(area => area.IsVehicleNotAtScene));
			}
		}
	}
}