﻿using System.Collections.Generic;
using ECrash.Core.Models.Document;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports
{
	public class ConditionInfo : DocumentChild<CrashReport>
	{
		private DistractionAction _distractionAction;
		private string _distractionActionDescription;
		private DistractionSource _distractionSource;
		private string _distractionSourceDescription;
		private string _conditionsDescription;
		private AlcoholUseSuspicion _alcoholUseSuspicion;
		private AlcoholTestStatus _alcoholTestStatus;
		private AlcoholTestType _alcoholTestType;
		private string _alcoholTestTypeDescription;
		private AlcoholTestResult _alcoholTestResult;
		private decimal? _bloodAlcoholContent;
		private bool _isAlcoholKitNumberUnknown;
		private string _alcoholKitNumber;
		private DrugUseSuspicion _drugUseSuspicion;
		private DrugTestStatus _drugTestStatus;
		private DrugTestType _drugTestType;
		private string _drugTestTypeDescription;
		private string _drugTestResultsDescription;
		private bool _isDrugKitNumberUnknown;
		private string _drugKitNumber;

		public ConditionInfo()
		{
			Conditions = new ConditionList().InitParent(this);
			DrugTestResults = new DrugTestResultList().InitParent(this);
		}

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			yield return Conditions;
			yield return DrugTestResults;
		}

		public DistractionAction DistractionAction
		{
			get => _distractionAction;
			set => Set(ref _distractionAction, value);
		}

		[MaxLength(200)]
		public string DistractionActionDescription
		{
			get => _distractionActionDescription;
			set => Set(ref _distractionActionDescription, value);
		}

		public DistractionSource DistractionSource
		{
			get => _distractionSource;
			set => Set(ref _distractionSource, value);
		}

		[MaxLength(200)]
		public string DistractionSourceDescription
		{
			get => _distractionSourceDescription;
			set => Set(ref _distractionSourceDescription, value);
		}

		[ListCannotContainNull]
		public ConditionList Conditions { get; }

		[MaxLength(200)]
		public string ConditionsDescription
		{
			get => _conditionsDescription;
			set => Set(ref _conditionsDescription, value);
		}

		public AlcoholUseSuspicion AlcoholUseSuspicion
		{
			get => _alcoholUseSuspicion;
			set => Set(ref _alcoholUseSuspicion, value);
		}

		public AlcoholTestStatus AlcoholTestStatus
		{
			get => _alcoholTestStatus;
			set => Set(ref _alcoholTestStatus, value);
		}

		public AlcoholTestType AlcoholTestType
		{
			get => _alcoholTestType;
			set => Set(ref _alcoholTestType, value);
		}

		[MaxLength(200)]
		public string AlcoholTestTypeDescription
		{
			get => _alcoholTestTypeDescription;
			set => Set(ref _alcoholTestTypeDescription, value);
		}

		public AlcoholTestResult AlcoholTestResult
		{
			get => _alcoholTestResult;
			set => Set(ref _alcoholTestResult, value);
		}

		public decimal? BloodAlcoholContent
		{
			get => _bloodAlcoholContent;
			set => Set(ref _bloodAlcoholContent, value);
		}

		public bool IsAlcoholKitNumberUnknown
		{
			get => _isAlcoholKitNumberUnknown;
			set => Set(ref _isAlcoholKitNumberUnknown, value);
		}

		[MaxLength(25)]
		public string AlcoholKitNumber
		{
			get => _alcoholKitNumber;
			set => Set(ref _alcoholKitNumber, value);
		}

		public DrugUseSuspicion DrugUseSuspicion
		{
			get => _drugUseSuspicion;
			set => Set(ref _drugUseSuspicion, value);
		}

		public DrugTestStatus DrugTestStatus
		{
			get => _drugTestStatus;
			set => Set(ref _drugTestStatus, value);
		}

		public DrugTestType DrugTestType
		{
			get => _drugTestType;
			set => Set(ref _drugTestType, value);
		}

		[MaxLength(200)]
		public string DrugTestTypeDescription
		{
			get => _drugTestTypeDescription;
			set => Set(ref _drugTestTypeDescription, value);
		}

		[ListCannotContainNull]
		public DrugTestResultList DrugTestResults { get; }

		[MaxLength(200)]
		public string DrugTestResultsDescription
		{
			get => _drugTestResultsDescription;
			set => Set(ref _drugTestResultsDescription, value);
		}

		public bool IsDrugKitNumberUnknown
		{
			get => _isDrugKitNumberUnknown;
			set => Set(ref _isDrugKitNumberUnknown, value);
		}

		[MaxLength(25)]
		public string DrugKitNumber
		{
			get => _drugKitNumber;
			set => Set(ref _drugKitNumber, value);
		}
	}
}