﻿using ECrash.Core.Models.Document;
using JetBrains.Annotations;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	public class Diagram : AttachmentBase<CrashReport>, INeedsCollectionIndex
	{
		[JsonConstructor]
		protected Diagram(Hash hash, int length) : base(hash, length)
		{
		}

		public Diagram([NotNull] byte[] data) : base(data)
		{
		}

		public override string AttachmentId => $"{Root?.Id:N}/diagrams/{Hash}/{Filename}";

		/// <summary>
		/// Gets or sets the zero-based index of this diagram.
		/// </summary>
		[JsonIgnore]
		public int Index => (Parent as DiagramList)?.IndexOf(this) ?? -1;

		/// <summary>
		/// Gets the one-based number of this diagram.
		/// </summary>
		[JsonIgnore]
		public int Number => Index + 1;

		public override Filename Filename => new($"Diagram{Index:00}.svg");

		public override string Type
		{
			get => "Diagram";
			set { }
		}

		public override string Description
		{
			get => $"Diagram {Number}";
			set { }
		}

		void INeedsCollectionIndex.OnCollectionIndexChanged()
		{
			Refresh();
		}
	}
}