﻿using System.Collections.Generic;
using ECrash.Core.Models.Document;
using ECrashLA.Core.Models.CrashReports.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	public class Driver : Occupant, IHasConditionInfo
	{
		private DriverLicenseStatus _driverLicenseStatus;
		private State _driverLicenseState;
		private string _driverLicenseStateDescription;
		private string _driverLicenseNumber;
		private DriverLicenseClass _driverLicenseClass;
		private DriverLicenseType _driverLicenseType;
		private CommercialDriverLicenseStatus _commercialDriverLicenseStatus;
		private string _commercialDriverLicenseStatusDescription;
		private EndorsementCompliance _endorsementCompliance;
		private string _driverLicenseEndorsementsDescription;
		private string _driverLicenseRestrictionsDescription;
		private AlcoholInterlockPresence _alcoholInterlockPresence;
		private SpeedingRelation _speedingRelation;
		private string _driverActionsDescription;
		private DriverAvoidanceManeuver _avoidanceManeuver;
		private string _avoidanceManeuverDescription;
		private PreCollisionStability _preCollisionStability;
		private string _preCollisionStabilityDescription;
		private VisionObscurement _visionObscurement;
		private string _visionObscurementDescription;

		public Driver()
		{
			DriverLicenseEndorsements = new DriverLicenseEndorsementList().InitParent(this);
			DriverLicenseRestrictions = new DriverLicenseRestrictionList().InitParent(this);
			DriverActions = new DriverActionList().InitParent(this);
			ConditionInfo = new ConditionInfo().InitParent(this);
			Citations = new CitationList().InitParent(this);
		}

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			foreach (var child in base.EnumerateChildNodes())
			{
				yield return child;
			}
			yield return DriverLicenseEndorsements;
			yield return DriverLicenseRestrictions;
			yield return DriverActions;
			yield return ConditionInfo;
			yield return Citations;
		}

		[JsonIgnore]
		public override int Index => (Parent as Vehicle)?.Index ?? -1;

		public override Vehicle GetParentVehicle()
		{
			return (Vehicle) Parent;
		}

		public DriverLicenseStatus DriverLicenseStatus
		{
			get => _driverLicenseStatus;
			set => Set(ref _driverLicenseStatus, value);
		}

		public State DriverLicenseState
		{
			get => _driverLicenseState;
			set => Set(ref _driverLicenseState, value);
		}

		[MaxLength(200)]
		public string DriverLicenseStateDescription
		{
			get => _driverLicenseStateDescription;
			set => Set(ref _driverLicenseStateDescription, value);
		}

		[MaxLength(25)]
		public string DriverLicenseNumber
		{
			get => _driverLicenseNumber;
			set => Set(ref _driverLicenseNumber, value);
		}

		public DriverLicenseClass DriverLicenseClass
		{
			get => _driverLicenseClass;
			set => Set(ref _driverLicenseClass, value);
		}

		public DriverLicenseType DriverLicenseType
		{
			get => _driverLicenseType;
			set => Set(ref _driverLicenseType, value);
		}

		public CommercialDriverLicenseStatus CommercialDriverLicenseStatus
		{
			get => _commercialDriverLicenseStatus;
			set => Set(ref _commercialDriverLicenseStatus, value);
		}

		[MaxLength(200)]
		public string CommercialDriverLicenseStatusDescription
		{
			get => _commercialDriverLicenseStatusDescription;
			set => Set(ref _commercialDriverLicenseStatusDescription, value);
		}

		public EndorsementCompliance EndorsementCompliance
		{
			get => _endorsementCompliance;
			set => Set(ref _endorsementCompliance, value);
		}

		[ListCannotContainNull]
		public DriverLicenseEndorsementList DriverLicenseEndorsements { get; }

		[MaxLength(200)]
		public string DriverLicenseEndorsementsDescription
		{
			get => _driverLicenseEndorsementsDescription;
			set => Set(ref _driverLicenseEndorsementsDescription, value);
		}

		[ListCannotContainNull]
		public DriverLicenseRestrictionList DriverLicenseRestrictions { get; }

		[MaxLength(200)]
		public string DriverLicenseRestrictionsDescription
		{
			get => _driverLicenseRestrictionsDescription;
			set => Set(ref _driverLicenseRestrictionsDescription, value);
		}

		public AlcoholInterlockPresence AlcoholInterlockPresence
		{
			get => _alcoholInterlockPresence;
			set => Set(ref _alcoholInterlockPresence, value);
		}

		public SpeedingRelation SpeedingRelation
		{
			get => _speedingRelation;
			set => Set(ref _speedingRelation, value);
		}

		[ListCannotContainNull]
		public DriverActionList DriverActions { get; }

		[MaxLength(200)]
		public string DriverActionsDescription
		{
			get => _driverActionsDescription;
			set => Set(ref _driverActionsDescription, value);
		}

		public DriverAvoidanceManeuver AvoidanceManeuver
		{
			get => _avoidanceManeuver;
			set => Set(ref _avoidanceManeuver, value);
		}

		[MaxLength(200)]
		public string AvoidanceManeuverDescription
		{
			get => _avoidanceManeuverDescription;
			set => Set(ref _avoidanceManeuverDescription, value);
		}

		public PreCollisionStability PreCollisionStability
		{
			get => _preCollisionStability;
			set => Set(ref _preCollisionStability, value);
		}

		[MaxLength(200)]
		public string PreCollisionStabilityDescription
		{
			get => _preCollisionStabilityDescription;
			set => Set(ref _preCollisionStabilityDescription, value);
		}

		public VisionObscurement VisionObscurement
		{
			get => _visionObscurement;
			set => Set(ref _visionObscurement, value);
		}

		[MaxLength(200)]
		public string VisionObscurementDescription
		{
			get => _visionObscurementDescription;
			set => Set(ref _visionObscurementDescription, value);
		}

		public ConditionInfo ConditionInfo { get; }

		[ListCannotContainNull]
		public CitationList Citations { get; }
	}
}