﻿using System;
using System.Text.RegularExpressions;
using ECrash.Core;

namespace ECrashLA.Core.Models.CrashReports;

public class LocalCaseNumberFormat : IEquatable<LocalCaseNumberFormat>
{
	private readonly Regex _dotnetRegex;

	public LocalCaseNumberFormat(int id, RowVersion rowVersion, string description, string dotnetRegex, string actiproRegex)
	{
		Id = id;
        RowVersion = rowVersion;
		Description = description;
		ActiproRegex = actiproRegex;
		DotnetRegex = dotnetRegex;
		_dotnetRegex = dotnetRegex == null ? null : new Regex(dotnetRegex);
	}

	public int Id { get; }
    public RowVersion RowVersion { get; }
	public string Description { get; }
	public string DotnetRegex { get; }
	public string ActiproRegex { get; }

	public bool IsMatch(string caseNumber)
	{
		return _dotnetRegex?.IsMatch(caseNumber) ?? false;
	}

	public bool Equals(LocalCaseNumberFormat other)
	{
		if (ReferenceEquals(null, other)) return false;
		if (ReferenceEquals(this, other)) return true;
		return Id == other.Id;
	}

	public override bool Equals(object obj)
	{
		if (ReferenceEquals(null, obj)) return false;
		if (ReferenceEquals(this, obj)) return true;
		if (obj.GetType() != this.GetType()) return false;
		return Equals((LocalCaseNumberFormat)obj);
	}

	public override int GetHashCode()
	{
		return Id;
	}

	public static bool operator ==(LocalCaseNumberFormat left, LocalCaseNumberFormat right)
	{
		return Equals(left, right);
	}

	public static bool operator !=(LocalCaseNumberFormat left, LocalCaseNumberFormat right)
	{
		return !Equals(left, right);
	}
}