﻿using System;
using System.Collections.Generic;
using System.Linq;
using ECrash.Core.Models.Document;
using ECrashLA.Core.Models.CrashReports.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	public class NonMotorist : Person, IHasConditionInfo, ICrashUnit
	{
		private Guid? _strikingVehicleId;
		private NonMotoristType _type;
		private string _typeDescription;
		private NonMotoristActionPriorToCrash _actionPriorToCrash;
		private string _actionPriorToCrashDescription;
		private string _contributingCircumstancesDescription;
		private NonMotoristOriginOrDestination _originOrDestination;
		private NonMotoristLocation _location;
		private string _locationDescription;
		private string _safetyEquipmentDescription;
		private ClothingBrightness _upperClothingBrightness;
		private ClothingBrightness _lowerClothingBrightness;
		private NonMotoristInitialContactPoint _initialContactPoint;

		public NonMotorist()
		{
			ConditionInfo = new ConditionInfo().InitParent(this);
			ContributingCircumstances = new NonMotoristContributingCircumstanceList().InitParent(this);
			SafetyEquipment = new NonMotoristSafetyEquipmentList().InitParent(this);
		}

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			yield return ConditionInfo;
			yield return ContributingCircumstances;
			yield return SafetyEquipment;
		}

		[JsonIgnore]
		public override int Index => (Parent as NonMotoristList)?.IndexOf(this) ?? -1;

		public ConditionInfo ConditionInfo { get; }

		public Guid? StrikingVehicleId
		{
			get => _strikingVehicleId;
			set => SetWithDependencies(ref _strikingVehicleId, value, nameof(StrikingVehicleId), nameof(StrikingVehicle));
		}

		[JsonIgnore]
		public Vehicle StrikingVehicle
		{
			get => Root?.Vehicles?.FirstOrDefault(x => x.NodeId == _strikingVehicleId);
			set => StrikingVehicleId = value?.NodeId;
		}

		public NonMotoristType Type
		{
			get => _type;
			set => Set(ref _type, value);
		}

		[MaxLength(200)]
		public string TypeDescription
		{
			get => _typeDescription;
			set => Set(ref _typeDescription, value);
		}

		public NonMotoristActionPriorToCrash ActionPriorToCrash
		{
			get => _actionPriorToCrash;
			set => Set(ref _actionPriorToCrash, value);
		}

		[MaxLength(200)]
		public string ActionPriorToCrashDescription
		{
			get => _actionPriorToCrashDescription;
			set => Set(ref _actionPriorToCrashDescription, value);
		}

		public NonMotoristOriginOrDestination OriginOrDestination
		{
			get => _originOrDestination;
			set => Set(ref _originOrDestination, value);
		}

		[ListCannotContainNull]
		public NonMotoristContributingCircumstanceList ContributingCircumstances { get; }

		[MaxLength(200)]
		public string ContributingCircumstancesDescription
		{
			get => _contributingCircumstancesDescription;
			set => Set(ref _contributingCircumstancesDescription, value);
		}

		public NonMotoristLocation Location
		{
			get => _location;
			set => Set(ref _location, value);
		}

		[MaxLength(200)]
		public string LocationDescription
		{
			get => _locationDescription;
			set => Set(ref _locationDescription, value);
		}

		[ListCannotContainNull]
		public NonMotoristSafetyEquipmentList SafetyEquipment { get; }

		[MaxLength(200)]
		public string SafetyEquipmentDescription
		{
			get => _safetyEquipmentDescription;
			set => Set(ref _safetyEquipmentDescription, value);
		}

		public ClothingBrightness UpperClothingBrightness
		{
			get => _upperClothingBrightness;
			set => Set(ref _upperClothingBrightness, value);
		}

		public ClothingBrightness LowerClothingBrightness
		{
			get => _lowerClothingBrightness;
			set => Set(ref _lowerClothingBrightness, value);
		}

		public NonMotoristInitialContactPoint InitialContactPoint
		{
			get => _initialContactPoint;
			set => Set(ref _initialContactPoint, value);
		}
	}
}