﻿using System.Collections.Generic;
using ECrash.Core.Models.Document;
using ECrashLA.Core.Models.CrashReports.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	public class NonVehicularProperty : DocumentChild<CrashReport>, INeedsCollectionIndex
	{
		private NonVehicularPropertyType _propertyType;
		private string _propertyTypeDescription;
		private NonVehicularPropertyDamageSeverity _damageSeverity;
		private bool _isOwnerNameUnknown;
		private string _ownerName;
		private bool _isOwnerAddressUnknown;
		private string _ownerAddressStreet;
		private string _ownerAddressCity;
		private State _ownerAddressState;
		private string _ownerAddressStateDescription;
		private string _ownerAddressPostalCode;
		private bool _isOwnerPhoneNumberUnknown;
		private string _ownerPhoneNumber;

		/// <summary>
		/// Gets or sets the zero-based index of this vehicle.
		/// </summary>
		[JsonIgnore]
		public int Index => (Parent as NonVehicularPropertyList)?.IndexOf(this) ?? -1;

		/// <summary>
		/// Gets the one-based number of this vehicle.
		/// </summary>
		[JsonIgnore]
		public int Number => Index + 1;

		public NonVehicularPropertyType PropertyType
		{
			get => _propertyType;
			set => Set(ref _propertyType, value);
		}

		[MaxLength(200)]
		public string PropertyTypeDescription
		{
			get => _propertyTypeDescription;
			set => Set(ref _propertyTypeDescription, value);
		}

		public NonVehicularPropertyDamageSeverity DamageSeverity
		{
			get => _damageSeverity;
			set => Set(ref _damageSeverity, value);
		}

		public bool IsOwnerNameUnknown
		{
			get => _isOwnerNameUnknown;
			set => Set(ref _isOwnerNameUnknown, value);
		}

		[MaxLength(200)]
		public string OwnerName
		{
			get => _ownerName;
			set => Set(ref _ownerName, value);
		}

		public bool IsOwnerAddressUnknown
		{
			get => _isOwnerAddressUnknown;
			set => Set(ref _isOwnerAddressUnknown, value);
		}

		[MaxLength(150)]
		public string OwnerAddressStreet
		{
			get => _ownerAddressStreet;
			set => Set(ref _ownerAddressStreet, value);
		}

		[MaxLength(50)]
		public string OwnerAddressCity
		{
			get => _ownerAddressCity;
			set => Set(ref _ownerAddressCity, value);
		}

		public State OwnerAddressState
		{
			get => _ownerAddressState;
			set => Set(ref _ownerAddressState, value);
		}

		[MaxLength(200)]
		public string OwnerAddressStateDescription
		{
			get => _ownerAddressStateDescription;
			set => Set(ref _ownerAddressStateDescription, value);
		}

		[MaxLength(20)]
		public string OwnerAddressPostalCode
		{
			get => _ownerAddressPostalCode;
			set => Set(ref _ownerAddressPostalCode, value);
		}

		public bool IsOwnerPhoneNumberUnknown
		{
			get => _isOwnerPhoneNumberUnknown;
			set => Set(ref _isOwnerPhoneNumberUnknown, value);
		}

		[MaxLength(20)]
		public string OwnerPhoneNumber
		{
			get => _ownerPhoneNumber;
			set => Set(ref _ownerPhoneNumber, value);
		}

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			yield break;
		}

		public void OnCollectionIndexChanged()
		{
			NotifyOfPropertyChange(nameof(Index));
			NotifyOfPropertyChange(nameof(Number));
		}
	}
}