﻿using System.Collections.Generic;
using ECrash.Core.Models.Document;
using ECrashLA.Core.Models.CrashReports.ReferenceData;

namespace ECrashLA.Core.Models.CrashReports
{
	public abstract class Occupant : Person
	{
		private SeatingPosition _seatingPosition;
		private RestraintSystemType _restraintSystemType;
		private string _restraintSystemTypeDescription;
		private ImproperRestraintSystemUsage _improperRestraintSystemUsage;
		private string _airBagDeploymentsDescription;
		private Ejection _ejection;
		private Extrication _extrication;

		protected Occupant()
		{
			AirBagDeployments = new AirBagDeploymentList().InitParent(this);
		}

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			yield return AirBagDeployments;
		}

		public SeatingPosition SeatingPosition
		{
			get => _seatingPosition;
			set => Set(ref _seatingPosition, value);
		}

		public RestraintSystemType RestraintSystemType
		{
			get => _restraintSystemType;
			set => Set(ref _restraintSystemType, value);
		}

		[MaxLength(200)]
		public string RestraintSystemTypeDescription
		{
			get => _restraintSystemTypeDescription;
			set => Set(ref _restraintSystemTypeDescription, value);
		}

		public ImproperRestraintSystemUsage ImproperRestraintSystemUsage
		{
			get => _improperRestraintSystemUsage;
			set => Set(ref _improperRestraintSystemUsage, value);
		}

		[ListCannotContainNull]
		public AirBagDeploymentList AirBagDeployments { get; }

		[MaxLength(200)]
		public string AirBagDeploymentsDescription
		{
			get => _airBagDeploymentsDescription;
			set => Set(ref _airBagDeploymentsDescription, value);
		}

		public Ejection Ejection
		{
			get => _ejection;
			set => Set(ref _ejection, value);
		}

		public Extrication Extrication
		{
			get => _extrication;
			set => Set(ref _extrication, value);
		}

		public abstract Vehicle GetParentVehicle();
	}
}