﻿using System;
using System.Collections.Generic;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class Agency : ReferenceDataBase
	{
		public const string TypeField = "Type";
		public const string SizeField = "Size";
		public const string TroopField = "Troop";
		public const string CasIdField = "CasId";
		public const string SupportedAttachmentExtensionsField = "SupportedAttachmentExtensions";
		public const string ChildAgenciesField = "ChildAgencies";

		public Agency(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public string Type
		{
			get => GetFieldValue<string>(TypeField);
			set => Fields[TypeField] = value;
		}

		[JsonIgnore]
		public int Size
		{
			get => GetFieldValue<int>(SizeField, 0);
			set => Fields[SizeField] = value;
		}

		[JsonIgnore]
		public string Troop
		{
			get => GetFieldValue<string>(TroopField);
			set => Fields[TroopField] = value;
		}

		[JsonIgnore]
		public Guid CasId
		{
			get => GetFieldObject<Guid>(CasIdField);
			set => Fields[CasIdField] = new JValue(value);
		}

		[JsonIgnore]
		public IEnumerable<string> SupportedAttachmentExtensions
		{
			get => GetFieldValues<string>(SupportedAttachmentExtensionsField);
			set => Fields[SupportedAttachmentExtensionsField] = JArray.FromObject(value ?? Array.Empty<string>());
		}

		[JsonIgnore]
		public IEnumerable<Guid> ChildAgencies
		{
			get => GetFieldValues<Guid>(ChildAgenciesField);
			set => Fields[ChildAgenciesField] = JArray.FromObject(value ?? Array.Empty<Guid>());
		}
	}
}