﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// The manner in which two motor vehicles in transport came together without regard
	/// to the direction of force, as defined by MMUCC field C9.
	/// </summary>
	public class CrashManner : ReferenceDataBase
	{
		// Tag definitions
		public const string BackingTag = "Backing";
		public const string CollisionBetweenTwoMotorVehiclesTag = "CollisionBetweenTwoMotorVehicles";
		public const string LeftTag = "Left";
		public const string OppositeDirectionTag = "OppositeDirection";
		public const string PassingTag = "Passing";
		public const string PerpendicularDirectionTag = "PerpendicularDirection";
		public const string RightTag = "Right";
		public const string SameDirectionTag = "SameDirection";
		
		// Field definitions
		public const string HelpTextField = nameof(HelpText);
		public const string HelpTopicField = nameof(HelpTopic);

		public CrashManner(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsBacking
		{
			get => Tags.Contains(BackingTag);
			set => SetTagPresence(BackingTag, value);
		}

		[JsonIgnore]
		public bool IsCollisionBetweenTwoMotorVehicles
		{
			get => Tags.Contains(CollisionBetweenTwoMotorVehiclesTag);
			set => SetTagPresence(CollisionBetweenTwoMotorVehiclesTag, value);
		}

		[JsonIgnore]
		public bool IsLeft
		{
			get => Tags.Contains(LeftTag);
			set => SetTagPresence(LeftTag, value);
		}

		[JsonIgnore]
		public bool IsOppositeDirection
		{
			get => Tags.Contains(OppositeDirectionTag);
			set => SetTagPresence(OppositeDirectionTag, value);
		}

		[JsonIgnore]
		public bool IsPassing
		{
			get => Tags.Contains(PassingTag);
			set => SetTagPresence(PassingTag, value);
		}

		[JsonIgnore]
		public bool IsPerpendicularDirection
		{
			get => Tags.Contains(PerpendicularDirectionTag);
			set => SetTagPresence(PerpendicularDirectionTag, value);
		}

		[JsonIgnore]
		public bool IsRight
		{
			get => Tags.Contains(RightTag);
			set => SetTagPresence(RightTag, value);
		}

		[JsonIgnore]
		public bool IsSameDirection
		{
			get => Tags.Contains(SameDirectionTag);
			set => SetTagPresence(SameDirectionTag, value);
		}

		[JsonIgnore]
		public string HelpText
		{
			get => GetFieldValue<string>(HelpTextField);
			set => Fields[HelpTextField] = value;
		}

		[JsonIgnore]
		public string HelpTopic
		{
			get => GetFieldValue<string>(HelpTopicField);
			set => Fields[HelpTopicField] = value;
		}
	}
}