﻿using System;
using System.Collections.Generic;
using System.Linq;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class DriverLicenseClass : ReferenceDataBase
	{
		// Tag definitions
		public const string CommercialTag = "Commercial";
		public const string MotorcycleTag = "Motorcycle";
		public const string AlwaysCommercialTag = "AlwaysCommercial";

		// Field definitions
		public const string IncludedStatesKey = "IncludedStates";
		public const string ExcludedStatesKey = "ExcludedStates";
		public const string RegexKey = "Regex";

		public DriverLicenseClass(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsCommercial
		{
			get => Tags.Contains(CommercialTag);
			set => SetTagPresence(CommercialTag, value);
		}

		[JsonIgnore]
		public bool IsMotorcycle
		{
			get => Tags.Contains(MotorcycleTag);
			set => SetTagPresence(MotorcycleTag, value);
		}

		[JsonIgnore]
		public bool IsAlwaysCommercial
		{
			get => Tags.Contains(AlwaysCommercialTag);
			set => SetTagPresence(AlwaysCommercialTag, value);
		}

		[JsonIgnore]
		public IEnumerable<string> IncludedStates
		{
			get => GetFieldValues<string>(IncludedStatesKey);
			set => Fields[IncludedStatesKey] = new JArray(value);
		}

		[JsonIgnore]
		public IEnumerable<string> ExcludedStates
		{
			get => GetFieldValues<string>(ExcludedStatesKey);
			set => Fields[ExcludedStatesKey] = new JArray(value);
		}

		public bool IsValidForState(string code)
		{
			if (string.IsNullOrWhiteSpace(code)) throw new ArgumentNullException(nameof(code));

			var includedStates = IncludedStates.ToList();
			var excludedStates = ExcludedStates.ToList();
			if (includedStates.Count > 0)
			{
				return includedStates.Contains(code);
			}
			if (excludedStates.Count > 0)
			{
				return !excludedStates.Contains(code);
			}
			return true;
		}

		public bool IsValidForState(State state)
		{
			if (state == null) throw new ArgumentNullException(nameof(state));
			return IsValidForState(state.Code);
		}

		[JsonIgnore]
		public string Regex
		{
			get { return GetFieldValue<string>(RegexKey); }
			set { Fields[RegexKey] = value; }
		}

		public bool IsMatchFor(string status)
		{
			string regex = Regex;
			return regex != null && System.Text.RegularExpressions.Regex.IsMatch(status, regex);
		}
	}
}