﻿using System.Collections.Generic;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class Highway : ReferenceDataBase
	{
		public const string InterstateTag = "Interstate";
		public const string USHighwayTag = "USHighway";
		public const string StateTag = "State";
		public const string BypassCodesField = "BypassCodes";

		public Highway(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsInterstate
		{
			get => Tags.Contains(InterstateTag);
			set => SetTagPresence(InterstateTag, value);
		}

		[JsonIgnore]
		public bool IsUSHighway
		{
			get => Tags.Contains(USHighwayTag);
			set => SetTagPresence(USHighwayTag, value);
		}

		[JsonIgnore]
		public bool IsState
		{
			get => Tags.Contains(StateTag);
			set => SetTagPresence(StateTag, value);
		}

		[JsonIgnore]
		public IEnumerable<string> BypassCodes
		{
			get => GetFieldValues<string>(BypassCodesField);
			set => Fields[BypassCodesField] = value != null ? new JArray(value) : null;
		}
	}
}