﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// Identifies the crash's location with respect to presence in a junction or proximity to components typically
	/// in junction or interchange areas, as defined by MMUCC field C15.2.
	/// </summary>
	public class JunctionLocation : ReferenceDataBase
	{
		public const string IntersectionTag = "Intersection";

		[JsonIgnore]
		public bool IsIntersection
		{
			get => Tags.Contains(IntersectionTag);
			set => SetTagPresence(IntersectionTag, value);
		}

		public const string RampTag = "Ramp";

		[JsonIgnore]
		public bool IsRamp
		{
			get => Tags.Contains(RampTag);
			set => SetTagPresence(RampTag, value);
		}

		public const string TrafficControlDevicesTag = "TrafficControlDevices";

		[JsonIgnore]
		public bool HasTrafficControlDevices
		{
			get => Tags.Contains(TrafficControlDevicesTag);
			set => SetTagPresence(TrafficControlDevicesTag, value);
		}

		public JunctionLocation(int id, RowVersion version) : base(id, version)
		{
		}
	}
}