﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class LicensePlateType : ReferenceDataBase
	{
		public const string OutOfStateTag = "OutOfState";
		public const string LouisianaTag = "Louisiana";
		public const string SpecialPlateTag = "SpecialPlate";
		public const string TemporaryTag = "Temporary";
		public const string PersonalizedTag = "Personalized";

		public LicensePlateType(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsOutOfState
		{
			get => Tags.Contains(OutOfStateTag);
			set => SetTagPresence(OutOfStateTag, value);
		}

		[JsonIgnore]
		public bool IsLouisiana
		{
			get => Tags.Contains(LouisianaTag);
			set => SetTagPresence(LouisianaTag, value);
		}

		[JsonIgnore]
		public bool IsSpecialPlate
		{
			get => Tags.Contains(SpecialPlateTag);
			set => SetTagPresence(SpecialPlateTag, value);
		}

		[JsonIgnore]
		public bool IsTemporaryTag
		{
			get => Tags.Contains(TemporaryTag);
			set => SetTagPresence(TemporaryTag, value);
		}

		[JsonIgnore]
		public bool IsPersonalized
		{
			get => Tags.Contains(PersonalizedTag);
			set => SetTagPresence(PersonalizedTag, value);
		}

		public string FormatTagNumber(string tagNumber)
		{
			if (tagNumber == null) return null;
			if (!IsSpecialPlate) return tagNumber;
			return string.Concat(Code, tagNumber.PadLeft(8 - Code.Length, '0'));
		}
	}
}