﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// Type of non-motorist involved in a crash, as defined by MMUCC field P4.1.
	/// </summary>
	public class NonMotoristType : ReferenceDataBase
	{
		public const string CyclistTag = "Cyclist";
		public const string PedestrianTag = "Pedestrian";
		public const string NonMotorVehicleOccupantTag = "NonMotorVehicleOccupant";
		public const string DescriptionLabelField = "DescriptionLabel";

		public NonMotoristType(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsCyclist
		{
			get => Tags.Contains(CyclistTag);
			set => SetTagPresence(CyclistTag, value);
		}

		[JsonIgnore]
		public bool IsPedestrian
		{
			get => Tags.Contains(PedestrianTag);
			set => SetTagPresence(PedestrianTag, value);
		}

		[JsonIgnore]
		public bool IsNonMotorVehicleOccupant
		{
			get => Tags.Contains(NonMotorVehicleOccupantTag);
			set => SetTagPresence(NonMotorVehicleOccupantTag, value);
		}


		[JsonIgnore]
		public string DescriptionLabel
		{
			get => GetFieldValue(DescriptionLabelField, "Description");
			set => Fields[DescriptionLabelField] = value;
		}
	}
}