﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class NonVehicularPropertyType : ReferenceDataBase
	{
		public const string PrivatePropertyTag = "PrivateProperty";
		public const string StatePropertyTag = "StateProperty";
		public const string VehicleEventCodeKey = "VehicleEventCode";
		public const string OwnerAddressStreetKey = "OwnerAddressStreet";
		public const string OwnerAddressCityKey = "OwnerAddressCity";
		public const string OwnerAddressStateKey = "OwnerAddressState";
		public const string OwnerAddressPostalCodeKey = "OwnerAddressPostalCode";

		public NonVehicularPropertyType(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsPrivateProperty
		{
			get => Tags.Contains(PrivatePropertyTag);
			set => SetTagPresence(PrivatePropertyTag, value);
		}

		[JsonIgnore]
		public bool IsStateProperty
		{
			get => Tags.Contains(StatePropertyTag);
			set => SetTagPresence(StatePropertyTag, value);
		}

		[JsonIgnore]
		public string VehicleEventCode
		{
			get => GetFieldValue<string>(VehicleEventCodeKey);
			set => Fields[VehicleEventCodeKey] = value;
		}

		[JsonIgnore]
		public string OwnerAddressStreet
		{
			get => GetFieldValue<string>(OwnerAddressStreetKey);
			set => Fields[OwnerAddressStreetKey] = value;
		}

		[JsonIgnore]
		public string OwnerAddressCity
		{
			get => GetFieldValue<string>(OwnerAddressCityKey);
			set => Fields[OwnerAddressCityKey] = value;
		}

		[JsonIgnore]
		public string OwnerAddressState
		{
			get => GetFieldValue<string>(OwnerAddressStateKey);
			set => Fields[OwnerAddressStateKey] = value;
		}

		[JsonIgnore]
		public string OwnerAddressPostalCode
		{
			get => GetFieldValue<string>(OwnerAddressPostalCodeKey);
			set => Fields[OwnerAddressPostalCodeKey] = value;
		}
	}
}