﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// Defines the ownership of the land where the crash occurred, as defined by MMUCC field C2.1.
	/// </summary>
	public class PropertyOwnership : ReferenceDataBase
	{
		// Tag definitions
		public const string PublicTag = "Public";
		public const string PrivateTag = "Private";

		public PropertyOwnership(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsPublic
		{
			get => Tags.Contains(PublicTag);
			set => SetTagPresence(PublicTag, value);
		}

		[JsonIgnore]
		public bool IsPrivate
		{
			get => Tags.Contains(PrivateTag);
			set => SetTagPresence(PrivateTag, value);
		}
	}
}