﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class RoadClassification : ReferenceDataBase
	{
		public const string RoadTag = "Road";
		public const string HighwayTag = "Highway";
		public const string InterstateTag = "Interstate";
		public const string USHighwayTag = "USHighway";
		public const string StateTag = "State";
		public const string ParishTag = "Parish";
		public const string PrivateTag = "Private";
		public const string StateParkTag = "StatePark";
		public const string OffRoadTag = "OffRoad";
		public const string MilepostNumberRequiredTag = "MilepostNumberRequired";
		public const string MilepostNumberForbiddenTag = "MilepostNumberForbidden";

		public RoadClassification(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsRoad
		{
			get => Tags.Contains(RoadTag);
			set => SetTagPresence(RoadTag, value);
		}

		[JsonIgnore]
		public bool IsHighway
		{
			get => Tags.Contains(HighwayTag);
			set => SetTagPresence(HighwayTag, value);
		}

		[JsonIgnore]
		public bool IsInterstate
		{
			get => Tags.Contains(InterstateTag);
			set => SetTagPresence(InterstateTag, value);
		}

		[JsonIgnore]
		public bool IsUSHighway
		{
			get => Tags.Contains(USHighwayTag);
			set => SetTagPresence(USHighwayTag, value);
		}

		[JsonIgnore]
		public bool IsState
		{
			get => Tags.Contains(StateTag);
			set => SetTagPresence(StateTag, value);
		}

		[JsonIgnore]
		public bool IsParish
		{
			get => Tags.Contains(ParishTag);
			set => SetTagPresence(ParishTag, value);
		}

		[JsonIgnore]
		public bool IsPrivate
		{
			get => Tags.Contains(PrivateTag);
			set => SetTagPresence(PrivateTag, value);
		}

		[JsonIgnore]
		public bool IsStatePark
		{
			get => Tags.Contains(StateParkTag);
			set => SetTagPresence(StateParkTag, value);
		}

		[JsonIgnore]
		public bool IsOffRoad
		{
			get => Tags.Contains(OffRoadTag);
			set => SetTagPresence(OffRoadTag, value);
		}

		[JsonIgnore]
		public bool MilepostNumberRequired
		{
			get => Tags.Contains(MilepostNumberRequiredTag);
			set => SetTagPresence(MilepostNumberRequiredTag, value);
		}

		[JsonIgnore]
		public bool MilepostNumberForbidden
		{
			get => Tags.Contains(MilepostNumberForbiddenTag);
			set => SetTagPresence(MilepostNumberForbiddenTag, value);
		}

		public bool MatchesHighway(Highway highway)
        {
			if (highway == null)
				return false;

			return (highway.IsInterstate && IsInterstate
				|| highway.IsUSHighway && IsUSHighway
				|| highway.IsState && IsState);				
        }
	}
}