﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class SeatingPosition : ReferenceDataBase
	{
		public const string OnGridTag = "OnGrid";
		
		public const string FrontRowTag = "FrontRow";
		public const string SecondRowTag = "SecondRow";
		public const string ThirdRowTag = "ThirdRow";
		public const string FourthRowTag = "FourthRow";
		public const string OtherRowTag = "OtherRow";
		public const string UnknownRowTag = "UnknownRow";

		public const string LeftSeatTag = "LeftSeat";
		public const string MiddleSeatTag = "MiddleSeat";
		public const string RightSeatTag = "RightSeat";
		public const string UnknownSeatTag = "UnknownSeat";

		public const string ExteriorTag = "ExteriorSeat";

		public const string DriverTag = "Driver";
		public const string TruckTag = "Truck";
		public const string TrailerTag = "Trailer";

		public const string GridRowKey = "GridRow";
		public const string GridColumnKey = "GridColumn";

		public SeatingPosition(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsOnGrid
		{
			get => Tags.Contains(OnGridTag);
			set => SetTagPresence(OnGridTag, value);
		}

		[JsonIgnore]
		public bool IsFrontRow
		{
			get => Tags.Contains(FrontRowTag);
			set => SetTagPresence(FrontRowTag, value);
		}

		[JsonIgnore]
		public bool IsSecondRow
		{
			get => Tags.Contains(SecondRowTag);
			set => SetTagPresence(SecondRowTag, value);
		}

		[JsonIgnore]
		public bool IsThirdRow
		{
			get => Tags.Contains(ThirdRowTag);
			set => SetTagPresence(ThirdRowTag, value);
		}

		[JsonIgnore]
		public bool IsFourthRow
		{
			get => Tags.Contains(FourthRowTag);
			set => SetTagPresence(FourthRowTag, value);
		}

		[JsonIgnore]
		public bool IsOtherRow
		{
			get => Tags.Contains(OtherRowTag);
			set => SetTagPresence(OtherRowTag, value);
		}

		[JsonIgnore]
		public bool IsUnknownRow
		{
			get => Tags.Contains(UnknownRowTag);
			set => SetTagPresence(UnknownRowTag, value);
		}

		[JsonIgnore]
		public bool IsLeftSeat
		{
			get => Tags.Contains(LeftSeatTag);
			set => SetTagPresence(LeftSeatTag, value);
		}

		[JsonIgnore]
		public bool IsMiddleSeat
		{
			get => Tags.Contains(MiddleSeatTag);
			set => SetTagPresence(MiddleSeatTag, value);
		}

		[JsonIgnore]
		public bool IsRightSeat
		{
			get => Tags.Contains(RightSeatTag);
			set => SetTagPresence(RightSeatTag, value);
		}

		[JsonIgnore]
		public bool IsUnknownSeat
		{
			get => Tags.Contains(UnknownSeatTag);
			set => SetTagPresence(UnknownSeatTag, value);
		}

		[JsonIgnore]
		public bool IsExteriorSeat
		{
			get => Tags.Contains(ExteriorTag);
			set => SetTagPresence(ExteriorTag, value);
		}

		[JsonIgnore]
		public bool IsDriver
		{
			get => Tags.Contains(DriverTag);
			set => SetTagPresence(DriverTag, value);
		}

		[JsonIgnore]
		public bool IsTruck
		{
			get => Tags.Contains(TruckTag);
			set => SetTagPresence(TruckTag, value);
		}

		[JsonIgnore]
		public bool IsTrailer
		{
			get => Tags.Contains(TrailerTag);
			set => SetTagPresence(TrailerTag, value);
		}

		[JsonIgnore]
		public int? GridRow
		{
			get => GetFieldValue<int?>(GridRowKey);
			set => Fields[GridRowKey] = value;
		}

		[JsonIgnore]
		public int? GridColumn
		{
			get => GetFieldValue<int?>(GridColumnKey);
			set => Fields[GridColumnKey] = value;
		}
	}
}