﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// A type of traffic control device applicable to a motor vehicle at the crash location, as defined by MMUCC field V17.
	/// </summary>
	public class TrafficControlDeviceType : ReferenceDataBase
	{
		// Tag definitions
		public const string PavementMarkingTag = "PavementMarking";
		public const string PersonTag = "Person";
		public const string SignalTag = "Signal";
		public const string SignTag = "Sign";
		public const string RequiresTrafficSignalStatusTag = "RequiresTrafficSignalStatus";

		public TrafficControlDeviceType(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsPavementMarking
		{
			get => Tags.Contains(PavementMarkingTag);
			set => SetTagPresence(PavementMarkingTag, value);
		}

		[JsonIgnore]
		public bool IsPerson
		{
			get => Tags.Contains(PersonTag);
			set => SetTagPresence(PersonTag, value);
		}

		[JsonIgnore]
		public bool IsSignal
		{
			get => Tags.Contains(SignalTag);
			set => SetTagPresence(SignalTag, value);
		}

		[JsonIgnore]
		public bool IsSign
		{
			get => Tags.Contains(SignTag);
			set => SetTagPresence(SignTag, value);
		}

		[JsonIgnore]
		public bool RequiresTrafficSignalStatus
		{
			get => Tags.Contains(RequiresTrafficSignalStatusTag);
			set => SetTagPresence(RequiresTrafficSignalStatusTag, value);
		}
	}
}