﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// A region on a vehicle, as defined by MMUCC field V19.
	/// </summary>
	public class VehicleArea : ReferenceDataBase
	{
		// Tag definitions
		public const string ValidForInitialContactPointTag = "ValidForInitialContactPoint";
		public const string ValidForDamagedAreasTag = "ValidForDamagedAreas";
		public const string VehicleNotAtSceneTag = "VehicleNotAtScene";
		public const string NoDamageTag = "NoDamage";

		// Field definitions
		public const string ClockPointField = nameof(ClockPoint);

		public VehicleArea(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsValidForInitialContactPoint
		{
			get => Tags.Contains(ValidForInitialContactPointTag);
			set => SetTagPresence(ValidForInitialContactPointTag, value);
		}

		[JsonIgnore]
		public bool IsValidForDamagedAreas
		{
			get => Tags.Contains(ValidForDamagedAreasTag);
			set => SetTagPresence(ValidForDamagedAreasTag, value);
		}

		/// <summary>
		/// Tag marks a value that is consistent with the VehicleDamageExtent value 'Vehicle Not At Scene'.
		/// </summary>
		[JsonIgnore]
		public bool IsVehicleNotAtScene
		{
			get => Tags.Contains(VehicleNotAtSceneTag);
			set => SetTagPresence(VehicleNotAtSceneTag, value);
		}

		/// <summary>
		/// Tag marks a value that is consistent with the VehicleDamageExtent value 'No Damage'
		/// </summary>
		[JsonIgnore]
		public bool IsNoDamage
		{
			get => Tags.Contains(NoDamageTag);
			set => SetTagPresence(NoDamageTag, value);
		}

		[JsonIgnore]
		public int ClockPoint
		{
			get => GetFieldValue(ClockPointField, 0);
			set => Fields[ClockPointField] = value;
		}
	}
}