﻿using System.Collections.Generic;
using System.Linq;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// The level of automation systems present or engaged in a vehicle, as defined by MMUCC field DV1.
	/// <para>Values are based on SAE International's Levels of Driving Automation, which can be seen in MMUCC on page 124.</para>
	/// </summary>
	public class VehicleAutomationSystemLevel : ReferenceDataBase
	{
		public const string CanBeEngagedIfPresentKey = nameof(CanBeEngagedIfPresent);

		public VehicleAutomationSystemLevel(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public IEnumerable<string> CanBeEngagedIfPresent
		{
			get => GetFieldValues<string>(CanBeEngagedIfPresentKey);
			set
			{
				var array = new JArray();
				if (value != null)
				{
					foreach (string code in value)
					{
						array.Add(code);
					}
				}
				Fields[CanBeEngagedIfPresentKey] = array;
			}
		}

		public bool CanBeEngaged(VehicleAutomationSystemLevel automationSystemLevelPresent)
		{
			if (automationSystemLevelPresent == null) return true;
			return CanBeEngagedIfPresent.Contains(automationSystemLevelPresent.Code);
		}
	}
}