﻿using System;
using System.Collections.Generic;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
    /// <summary>
    /// The motor vehicle body type as defined by MMUCC field V8.1.
    /// </summary>
	public class VehicleBodyType : ReferenceDataBase
	{
		public const string BusUsageTag = "BusUsage";
		public const string CycleOrLowSpeedTag = "CycleOrLowSpeed";
		public const string EquipmentTag = "Equipment";
		public const string LargePassengerVehicleTag = "LargePassengerVehicle";
		public const string LargeVehicleTag = "LargeVehicle";
		public const string MotorcycleTag = "Motorcycle";
		public const string PassengerVehicleTag = "PassengerVehicle";
		public const string SchoolBusTag = "SchoolBus";
		public const string TruckTag = "Truck";
		public const string NoEjectionTag = "NoEjection";
		public const string NhtsaValueIdsField = nameof(NhtsaValueIds);
		public const string HelpTopicField = nameof(HelpTopic);

		public VehicleBodyType(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsPassengerVehicle
		{
			get => Tags.Contains(PassengerVehicleTag);
			set => SetTagPresence(PassengerVehicleTag, value);
		}

		[JsonIgnore]
		public bool IsLargeVehicle
		{
			get => Tags.Contains(LargeVehicleTag);
			set => SetTagPresence(LargeVehicleTag, value);
		}

		[JsonIgnore]
		public bool IsEquipment
		{
			get => Tags.Contains(EquipmentTag);
			set => SetTagPresence(EquipmentTag, value);
		}

		[JsonIgnore]
		public bool IsMotorcycle
		{
			get => Tags.Contains(MotorcycleTag);
			set => SetTagPresence(MotorcycleTag, value);
		}

		[JsonIgnore]
		public bool IsCycleOrLowSpeed
		{
			get => Tags.Contains(CycleOrLowSpeedTag);
			set => SetTagPresence(CycleOrLowSpeedTag, value);
		}

		[JsonIgnore]
		public bool IsTruck
		{
			get => Tags.Contains(TruckTag);
			set => SetTagPresence(TruckTag, value);
		}

		[JsonIgnore]
		public bool IsLargePassengerVehicle
		{
			get => Tags.Contains(LargePassengerVehicleTag);
			set => SetTagPresence(LargePassengerVehicleTag, value);
		}

		[JsonIgnore]
		public bool IsNoEjection
		{
			get => Tags.Contains(NoEjectionTag);
			set => SetTagPresence(NoEjectionTag, value);
		}

		/// <summary>
		/// Marks body types that require a bus-related VehicleUsage value, according to MMUCC edit check V10.02.
		/// </summary>
		[JsonIgnore]
		public bool IsBusUsage
		{
			get => Tags.Contains(BusUsageTag);
			set => SetTagPresence(BusUsageTag, value);
		}

		[JsonIgnore]
		public bool IsSchoolBus
		{
			get => Tags.Contains(SchoolBusTag);
			set => SetTagPresence(SchoolBusTag, value);
		}

		/// <summary>
		/// Value IDs used by the National Highway Traffic Safety Administration for this body type.
		/// Used in VIN imports.
		/// </summary>
		[JsonIgnore]
		public IEnumerable<string> NhtsaValueIds
		{
			get => GetFieldValues<string>(NhtsaValueIdsField);
			set => Fields[NhtsaValueIdsField] = JArray.FromObject(value ?? Array.Empty<string>());
		}

		[JsonIgnore]
		public string HelpTopic
		{
			get => GetFieldValue<string>(HelpTopicField);
			set => Fields[HelpTopicField] = value;
		}
	}
}