﻿using System;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class VehicleColor : ReferenceDataBase
	{
		public const string NcicCodeField = "NcicCode";
		public const string TranciteColorField = "TranciteColor";

		public VehicleColor(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public string NcicCode
		{
			get => GetFieldValue<string>(NcicCodeField);
			set => Fields[NcicCodeField] = value;
		}

		[JsonIgnore]
		public string TranciteColor
		{
			get => GetFieldValue<string>(TranciteColorField);
			set => Fields[TranciteColorField] = value;
		}

		/// <summary>
		/// Returns true if <paramref name="color"/> matches this color's name or its NCIC code.
		/// </summary>
		/// <param name="color"></param>
		/// <returns></returns>
		public bool IsMatchByNameOrNcicCode(string color)
		{
			if (string.IsNullOrWhiteSpace(color))
			{
				return false;
			}

			if (Name.Equals(color, StringComparison.InvariantCultureIgnoreCase))
			{
				return true;
			}

			if (NcicCode?.Equals(color, StringComparison.InvariantCultureIgnoreCase) == true)
			{
				return true;
			}

			return false;
		}
	}
}