﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class VehicleEvent : ReferenceDataBase
	{
		// Tag definitions
		public const string HarmfulTag = "Harmful";
		public const string CollisionTag = "Collision";
		public const string FixedObjectTag = "FixedObject";
		public const string RoadBoundaryTag = "RoadBoundary";
		public const string NonMotoristTag = "NonMotorist";
		public const string CyclistTag = "Cyclist";
		public const string PedestrianTag = "Pedestrian";
		public const string MotorVehicleInTransportTag = "MotorVehicleInTransport";
		public const string ParkedMotorVehicleTag = "ParkedMotorVehicle";
		public const string TrafficControlSupportTag = "TrafficControlSupport";
		public const string CrossCenterlineTag = "CrossCenterline";
		public const string CrossMedianTag = "CrossMedian";
		public const string WorkZoneEquipmentTag = "WorkZoneEquipment";
		public const string ObjectFromVehicleTag = "ObjectFromVehicle";

		// Category definitions
		public const string NonCollisionCategory = "Non-Collision";
		public const string CollisionWithNonFixedObjectCategory = "Collision With Person, Motor Vehicle, or Non-Fixed Object";
		public const string CollisionWithFixedObjectCategory = "Collision With Fixed Object";

		public VehicleEvent(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsHarmful
		{
			get => Tags.Contains(HarmfulTag);
			set => SetTagPresence(HarmfulTag, value);
		}

		[JsonIgnore]
		public bool IsCollision
		{
			get => Tags.Contains(CollisionTag);
			set => SetTagPresence(CollisionTag, value);
		}

		[JsonIgnore]
		public bool IsFixedObject
		{
			get => Tags.Contains(FixedObjectTag);
			set => SetTagPresence(FixedObjectTag, value);
		}

		[JsonIgnore]
		public bool IsRoadBoundary
		{
			get => Tags.Contains(RoadBoundaryTag);
			set => SetTagPresence(RoadBoundaryTag, value);
		}

		[JsonIgnore]
		public bool IsNonMotorist
		{
			get => Tags.Contains(NonMotoristTag);
			set => SetTagPresence(NonMotoristTag, value);
		}

		[JsonIgnore]
		public bool IsCyclist
		{
			get => Tags.Contains(CyclistTag);
			set => SetTagPresence(CyclistTag, value);
		}

		[JsonIgnore]
		public bool IsPedestrian
		{
			get => Tags.Contains(PedestrianTag);
			set => SetTagPresence(PedestrianTag, value);
		}

		[JsonIgnore]
		public bool IsMotorVehicleInTransport
		{
			get => Tags.Contains(MotorVehicleInTransportTag);
			set => SetTagPresence(MotorVehicleInTransportTag, value);
		}

		[JsonIgnore]
		public bool IsParkedMotorVehicle
		{
			get => Tags.Contains(ParkedMotorVehicleTag);
			set => SetTagPresence(ParkedMotorVehicleTag, value);
		}

		[JsonIgnore]
		public bool IsWorkZoneEquipment
		{
			get => Tags.Contains(WorkZoneEquipmentTag);
			set => SetTagPresence(WorkZoneEquipmentTag, value);
		}

		[JsonIgnore]
		public bool IsObjectFromVehicle
		{
			get => Tags.Contains(ObjectFromVehicleTag);
			set => SetTagPresence(ObjectFromVehicleTag, value);
		}

		[JsonIgnore]
		public bool IsTrafficControlSupport
		{
			get => Tags.Contains(TrafficControlSupportTag);
			set => SetTagPresence(TrafficControlSupportTag, value);
		}

		[JsonIgnore]
		public bool IsCrossCenterline
		{
			get => Tags.Contains(CrossCenterlineTag);
			set => SetTagPresence(CrossCenterlineTag, value);
		}

		[JsonIgnore]
		public bool IsCrossMedian
		{
			get => Tags.Contains(CrossMedianTag);
			set => SetTagPresence(CrossMedianTag, value);
		}
	}
}