﻿using System;
using System.Collections.Generic;
using System.Linq;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	public class VehicleMake : ReferenceDataBase
	{
		public const string AbbreviationField = nameof(Abbreviation);
		public const string FarsCodesField = nameof(FarsCode);
		public const string NhtsaValueIdsField = nameof(NhtsaValueIds);

		public VehicleMake(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public IEnumerable<string> Abbreviation
		{
			get
			{
				IEnumerable<string> abbrevs;
				try
				{
					abbrevs = GetFieldValues<string>(AbbreviationField);
				}
				catch (InvalidOperationException)
				{
					abbrevs = Array.Empty<string>();
				}

				return abbrevs;
			}
			set => Fields[AbbreviationField] = JArray.FromObject(value ?? Array.Empty<string>());
		}

		[JsonIgnore]
		public string FarsCode
		{
			get => GetFieldValue<string>(FarsCodesField);
			set => Fields[FarsCodesField] = value;
		}

		/// <summary>
		/// Value IDs used by the National Highway Traffic Safety Administration for this make.
		/// Used in VIN imports.
		/// </summary>
		[JsonIgnore]
		public IEnumerable<string> NhtsaValueIds
		{
			get => GetFieldValues<string>(NhtsaValueIdsField);
			set => Fields[NhtsaValueIdsField] = JArray.FromObject(value ?? Array.Empty<string>());
		}

		/// <summary>
		/// Returns true if <paramref name="make"/> matches this make's name or one of its abbreviations.
		/// </summary>
		/// <param name="make"></param>
		/// <returns></returns>
		public bool IsMatchByNameOrAbbreviation(string make)
		{
			if (string.IsNullOrWhiteSpace(make))
			{
				throw new ArgumentNullException();
			}

			if (Name.Equals(make, StringComparison.InvariantCultureIgnoreCase))
			{
				return true;
			}

			if (Abbreviation?.Any(x => x.Replace('0', 'O').Equals(make.Replace('0', 'O'), StringComparison.InvariantCultureIgnoreCase)) == true)
			{
				return true;
			}

			return false;
		}
	}
}