﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
    /// <summary>
    /// The direction of a motor vehicle's travel on the roadway before the crash as defined by MMUCC field V13.
    /// <para>
    /// This is not a compass direction, but a direction consistent with the designated direction of the road.
    /// </para>
    /// </summary>
	public class VehicleTravelDirection : ReferenceDataBase
    {
	    public const string ArrowKey = "Arrow";

	    [JsonIgnore]
	    public string Arrow
	    {
		    get => GetFieldValue<string>(ArrowKey);
		    set => Fields[ArrowKey] = value;
	    }

	    public const string NotOnTrafficwayTag = "NotOnTrafficway";

	    [JsonIgnore]
	    public bool IsNotOnTrafficway
	    {
		    get => Tags.Contains(NotOnTrafficwayTag);
		    set => SetTagPresence(NotOnTrafficwayTag, value);
	    }

		public VehicleTravelDirection(int id, RowVersion version) : base(id, version)
		{
		}
	}
}