﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
    /// <summary>
    /// Indicates the type of special function being served by this vehicle, regardless of whether the function is marked on the vehicle, at the time of the crash,
    /// as defined by MMUCC field V10.
    /// </summary>
	public class VehicleUsage : ReferenceDataBase
	{
		// Tag definitions
		public const string BusTag = "Bus";
		public const string SchoolBusTag = "SchoolBus";
		public const string EmergencyTag = "Emergency";

		public VehicleUsage(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsBus
		{
			get => Tags.Contains(BusTag);
			set => SetTagPresence(BusTag, value);
		}

		[JsonIgnore]
		public bool IsSchoolBus
		{
			get => Tags.Contains(SchoolBusTag);
			set => SetTagPresence(SchoolBusTag, value);
		}

		[JsonIgnore]
		public bool IsEmergency
		{
			get => Tags.Contains(EmergencyTag);
			set => SetTagPresence(EmergencyTag, value);
		}
	}
}