﻿using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
	/// <summary>
	/// Represents a motor vehicle-related violation code, or none, that applies to a driver, as defined by MMUCC field P15.
	/// </summary>
	public class ViolationCode : ReferenceDataBase
	{
		public const string DuiTag = "DUI";
		public const string CodeNumberKey = "CodeNumber";
		public const string ViolationCategoryKey = "ViolationCategory";

		public ViolationCode(int id, RowVersion version) : base(id, version)
		{
		}

		[JsonIgnore]
		public bool IsDui
		{
			get => Tags.Contains(DuiTag);
			set => SetTagPresence(DuiTag, value);
		}

		[JsonIgnore]
		public string CodeNumber
		{
			get => GetFieldValue<string>(CodeNumberKey);
			set => Fields[CodeNumberKey] = value;
		}

		[JsonIgnore]
		public string ViolationCategory
		{
			get => GetFieldValue<string>(ViolationCategoryKey);
			set => Fields[ViolationCategoryKey] = value;
		}
	}
}