﻿using System;
using System.Collections.Generic;
using ECrash.Core;
using ECrash.Core.Models.ReferenceData;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace ECrashLA.Core.Models.CrashReports.ReferenceData
{
    /// <summary>
    /// The vehicle's weight rating as defined by MMUCC V8.3.
    /// </summary>
	public class WeightRating : ReferenceDataBase
	{
		public const string LightWeightTag = "LightWeight";
		public const string MediumWeightTag = "MediumWeight";
		public const string HeavyWeightTag = "HeavyWeight";
        public const string NhtsaValueIdsField = nameof(NhtsaValueIds);

		[JsonIgnore]
		public bool IsLightWeight
		{
			get => Tags.Contains(LightWeightTag);
			set => SetTagPresence(LightWeightTag, value);
		}

		[JsonIgnore]
		public bool IsMediumWeight
		{
			get => Tags.Contains(MediumWeightTag);
			set => SetTagPresence(MediumWeightTag, value);
		}

		[JsonIgnore]
		public bool IsHeavyWeight
		{
			get => Tags.Contains(HeavyWeightTag);
			set => SetTagPresence(HeavyWeightTag, value);
		}

        /// <summary>
        /// Value IDs used by the National Highway Traffic Safety Administration for this weight rating (25: Gross Vehicle Weight Rating From).
        /// Used in VIN imports.
        /// </summary>
        [JsonIgnore]
        public IEnumerable<string> NhtsaValueIds
        {
            get => GetFieldValues<string>(NhtsaValueIdsField);
            set => Fields[NhtsaValueIdsField] = JArray.FromObject(value ?? Array.Empty<string>());
        }

		public WeightRating(int id, RowVersion version) : base(id, version)
		{
		}
	}
}