﻿using System.Collections.Generic;
using ECrash.Core.Models;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.JsonSchema;
using ECrashLA.Core.Models.CrashReports.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	/// <summary>
    /// Represents a motor vehicle involved in the crash being reported.
    /// </summary>
	public class Vehicle : DocumentChild<CrashReport>, INeedsCollectionIndex, IHasVehicleRegistration, ICrashUnit
	{
		private bool _isVinUnknown;
		private string _vin;
		private VehicleType _type;
		private bool _isTagMissing;
		private bool _isTagStateUnknown;
		private State _tagState;
		private string _tagStateDescription;
		private bool _isTagNumberUnknown;
		private string _tagNumber;
		private bool _isTagYearUnknown;
		private bool _isTagNonExpiring;
		private int? _tagYear;
		private VehicleMake _make;
		private string _makeDescription;
		private VehicleModel _model;
		private string _modelDescription;
		private bool _isModelYearUnknown;
		private int? _modelYear;
		private VehicleColor _color;
		private string _colorDescription;
		private bool _isOwnerNameSameAsDriver;
		private bool _isOwnerNameUnknown;
		private string _ownerName;
		private bool _isOwnerAddressSameAsDriver;
		private bool _isOwnerAddressUnknown;
		private string _ownerAddressStreet;
		private string _ownerAddressCity;
		private State _ownerAddressState;
		private string _ownerAddressStateDescription;
		private string _ownerAddressPostalCode;
		private bool _isUninsured;
		private bool _isLiabilityInsuranceCompanyUnknown;
		private string _liabilityInsuranceCompany;
		private bool _isLiabilityInsuranceCompanyPhoneNumberUnknown;
		private string _liabilityInsuranceCompanyPhoneNumber;
		private bool _isLiabilityInsuranceNaicNumberUnknown;
		private string _liabilityInsuranceNaicNumber;
		private bool _isLiabilityInsurancePolicyNumberUnknown;
		private string _liabilityInsurancePolicyNumber;
		private bool _isLiabilityInsurancePolicyExpirationDateUnknown;
		private Date? _liabilityInsurancePolicyExpirationDate;
		private VehicleBodyType _bodyType;
		private string _bodyTypeDescription;
		private WeightRating _weightRating;
		private HazardousMaterialsPlacardStatus _hazardousMaterialsPlacardStatus;
		private VehicleUsage _vehicleUsage;
		private string _vehicleUsageDescription;
		private EmergencyVehicleUsage _emergencyVehicleUsage;
		private bool _isSpeedLimitUnknown;
		private bool _isSpeedLimitNotApplicable;
		private int? _speedLimit;
		private VehicleTravelDirection _travelDirection;
		private TrafficwayTravelDirection _trafficwayTravelDirection;
		private TrafficwayDivision _trafficwayDivision;
		private TrafficwayBarrierType _trafficwayBarrierType;
		private string _trafficwayBarrierTypeDescription;
		private TrafficwayHovLanePresence _trafficwayHovLanePresence;
		private TrafficwayHovLaneRelation _trafficwayHovLaneRelation;
		private int? _totalThroughLanes;
		private int? _totalAuxiliaryLanes;
		private RoadwayAlignment _roadwayAlignment;
		private RoadwayGrade _roadwayGrade;
		private string _trafficControlDevicesPresentDescription;
		private TrafficSignalStatus _trafficSignalStatus;
		private string _trafficControlDevicesInoperativeOrMissingDescription;
		private VehicleManeuver _maneuver;
		private string _maneuverDescription;
		private VehicleManeuverReason _maneuverReason;
		private string _maneuverReasonDescription;
		private VehicleLighting _lighting;
		private VehicleArea _initialPointOfContact;
		private VehicleDamageExtent _damageExtent;
		private HitAndRun _hitAndRun;
		private TowStatus _towStatus;
		private bool _isTowServiceUnknown;
		private string _towService;
		private TowAuthority _towAuthority;
		private string _towAuthorityDescription;
		private VehicleContributingCircumstance _contributingCircumstance;
		private string _contributingCircumstanceDescription;
		private Driver _driver;
		private VehicleFmcsaReportability _fmcsaReportability;
		private bool _isNumberOfAxlesUnknown;
		private int? _numberOfAxles;
		private MotorCarrierIdentificationType _motorCarrierIdentificationType;
		private State _motorCarrierIdentificationState;
		private string _motorCarrierIdentificationStateDescription;
		private string _motorCarrierIdentificationNumber;
		private bool _isMotorCarrierNameUnknown;
		private string _motorCarrierName;
		private bool _isMotorCarrierAddressUnknown;
		private string _motorCarrierAddressStreet;
		private string _motorCarrierAddressCity;
		private State _motorCarrierAddressState;
		private string _motorCarrierAddressStateDescription;
		private string _motorCarrierAddressPostalCode;
		private bool _isMotorCarrierPhoneNumberUnknown;
		private string _motorCarrierPhoneNumber;
		private MotorCarrierType _motorCarrierType;
		private string _motorCarrierTypeDescription;
		private VehicleConfiguration _vehicleConfiguration;
		private VehicleLoadPermit _vehicleLoadPermit;
		private CargoBodyType _cargoBodyType;
		private string _cargoBodyTypeDescription;
		private bool _isCommodityHauledUnknown;
		private string _commodityHauled;
		private bool _isHazardousMaterialsIdUnknown;
		private bool _isHazardousMaterialsIdNotApplicable;
		private string _hazardousMaterialsId;
		private HazardousMaterialClass _hazardousMaterialClass;
		private HazardousMaterialRelease _hazardousMaterialRelease;
		private VehicleAutomationSystemLevel _automationSystemLevelPresent;
		private VehicleAutomationSystemLevel _automationSystemLevelEngaged;
		private bool _isDistanceTraveledAfterImpactUnknown;
		private int? _distanceTraveledAfterImpact;
		private bool _isSkidmarkLengthNotApplicable;
		private bool _isSkidmarkLengthUnknown;
		private int? _skidmarkLengthFrontLeft;
		private int? _skidmarkLengthFrontRight;
		private int? _skidmarkLengthRearLeft;
		private int? _skidmarkLengthRearRight;
		private LicensePlateType _licensePlateType;

		public Vehicle()
		{
			TrafficControlDevicesPresent = new TrafficControlDeviceTypeList().InitParent(this);
			TrafficControlDevicesInoperativeOrMissing = new TrafficControlDeviceTypeList().InitParent(this);
			DamagedAreas = new VehicleAreaList().InitParent(this);
			SequenceOfEvents = new SequenceOfEvents().InitParent(this);
			Trailers = new VehicleTrailerList().InitParent(this);
			_driver = new Driver().InitParent(this);
			Passengers = new PassengerList().InitParent(this);
			VehicleSpecialSizings = new VehicleSpecialSizingList().InitParent(this);
		}

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			yield return TrafficControlDevicesPresent;
			yield return TrafficControlDevicesInoperativeOrMissing;
			yield return DamagedAreas;
			yield return SequenceOfEvents;
			yield return Trailers;
			if (_driver != null) yield return _driver;
			yield return Passengers;
			yield return VehicleSpecialSizings;
		}

		void INeedsCollectionIndex.OnCollectionIndexChanged()
		{
			NotifyOfPropertyChange(nameof(Index));
			NotifyOfPropertyChange(nameof(Number));
			(_driver as INeedsCollectionIndex)?.OnCollectionIndexChanged();
		}

        /// <summary>
        /// Gets or sets the zero-based index of this vehicle.
        /// </summary>
		[JsonIgnore]
		public int Index => (Parent as VehicleList)?.IndexOf(this) ?? -1;

        /// <summary>
        /// Gets the one-based number of this vehicle.
        /// </summary>
		[JsonIgnore]
		public int Number => Index + 1;

		public bool IsVinUnknown
		{
			get => _isVinUnknown;
			set => Set(ref _isVinUnknown, value);
		}

        /// <summary>
        /// Gets or sets the vehicle identification number.
        /// </summary>
        [MaxLength(20)]
		public string Vin
		{
			get => _vin;
			set => Set(ref _vin, value);
		}

		public VehicleType Type
		{
			get => _type;
			set => Set(ref _type, value);
		}

		public bool IsTagMissing
		{
			get => _isTagMissing;
			set => Set(ref _isTagMissing, value);
		}

		public bool IsTagStateUnknown
		{
			get => _isTagStateUnknown;
			set => Set(ref _isTagStateUnknown, value);
		}

        /// <summary>
        /// Gets or sets the tag's registered state.
        /// </summary>
		public State TagState
		{
			get => _tagState;
			set => Set(ref _tagState, value);
		}

		[MaxLength(200)]
		public string TagStateDescription
		{
			get => _tagStateDescription;
			set => Set(ref _tagStateDescription, value);
		}

		public bool IsTagNumberUnknown
		{
			get => _isTagNumberUnknown;
			set => Set(ref _isTagNumberUnknown, value);
		}

		public bool IsTagNonExpiring
		{
			get => _isTagNonExpiring;
			set => Set(ref _isTagNonExpiring, value);
		}

		[MaxLength(10)]
		public string TagNumber
		{
			get => _tagNumber;
			set => Set(ref _tagNumber, value);
		}

		public bool IsTagYearUnknown
		{
			get => _isTagYearUnknown;
			set => Set(ref _isTagYearUnknown, value);
		}

        /// <summary>
        /// Gets or sets the year on the vehicle's license plate.
        /// </summary>
		public int? TagYear
		{
			get => _tagYear;
			set => Set(ref _tagYear, value);
		}

		public VehicleMake Make
		{
			get => _make;
			set => Set(ref _make, value);
		}

		[MaxLength(200)]
		public string MakeDescription
		{
			get => _makeDescription;
			set => Set(ref _makeDescription, value);
		}

		public VehicleModel Model
		{
			get => _model;
			set => Set(ref _model, value);
		}

		[MaxLength(200)]
		public string ModelDescription
		{
			get => _modelDescription;
			set => Set(ref _modelDescription, value);
		}

		public bool IsModelYearUnknown
		{
			get => _isModelYearUnknown;
			set => Set(ref _isModelYearUnknown, value);
		}

		public int? ModelYear
		{
			get => _modelYear;
			set => Set(ref _modelYear, value);
		}

		public VehicleColor Color
		{
			get => _color;
			set => Set(ref _color, value);
		}

		[MaxLength(200)]
		public string ColorDescription
		{
			get => _colorDescription;
			set => Set(ref _colorDescription, value);
		}

		public bool IsOwnerNameSameAsDriver
		{
			get => _isOwnerNameSameAsDriver;
			set => Set(ref _isOwnerNameSameAsDriver, value);
		}

		public bool IsOwnerNameUnknown
		{
			get => _isOwnerNameUnknown;
			set => Set(ref _isOwnerNameUnknown, value);
		}

		[MaxLength(200)]
		public string OwnerName
		{
			get => _ownerName;
			set => Set(ref _ownerName, value);
		}

		public bool IsOwnerAddressSameAsDriver
		{
			get => _isOwnerAddressSameAsDriver;
			set => Set(ref _isOwnerAddressSameAsDriver, value);
		}

		public bool IsOwnerAddressUnknown
		{
			get => _isOwnerAddressUnknown;
			set => Set(ref _isOwnerAddressUnknown, value);
		}

		[MaxLength(150)]
		public string OwnerAddressStreet
		{
			get => _ownerAddressStreet;
			set => Set(ref _ownerAddressStreet, value);
		}

		[MaxLength(50)]
		public string OwnerAddressCity
		{
			get => _ownerAddressCity;
			set => Set(ref _ownerAddressCity, value);
		}

		public State OwnerAddressState
		{
			get => _ownerAddressState;
			set => Set(ref _ownerAddressState, value);
		}

		[MaxLength(200)]
		public string OwnerAddressStateDescription
		{
			get => _ownerAddressStateDescription;
			set => Set(ref _ownerAddressStateDescription, value);
		}

		[MaxLength(20)]
		public string OwnerAddressPostalCode
		{
			get => _ownerAddressPostalCode;
			set => Set(ref _ownerAddressPostalCode, value);
		}

		public bool IsUninsured
		{
			get => _isUninsured;
			set => Set(ref _isUninsured, value);
		}

		public bool IsLiabilityInsuranceCompanyUnknown
		{
			get => _isLiabilityInsuranceCompanyUnknown;
			set => Set(ref _isLiabilityInsuranceCompanyUnknown, value);
		}

		[MaxLength(50)]
		public string LiabilityInsuranceCompany
		{
			get => _liabilityInsuranceCompany;
			set => Set(ref _liabilityInsuranceCompany, value);
		}

		public bool IsLiabilityInsuranceCompanyPhoneNumberUnknown
		{
			get => _isLiabilityInsuranceCompanyPhoneNumberUnknown;
			set => Set(ref _isLiabilityInsuranceCompanyPhoneNumberUnknown, value);
		}

		[MaxLength(20)]
		public string LiabilityInsuranceCompanyPhoneNumber
		{
			get => _liabilityInsuranceCompanyPhoneNumber;
			set => Set(ref _liabilityInsuranceCompanyPhoneNumber, value);
		}

		public bool IsLiabilityInsuranceNaicNumberUnknown
		{
			get => _isLiabilityInsuranceNaicNumberUnknown;
			set => Set(ref _isLiabilityInsuranceNaicNumberUnknown, value);
		}

		[MaxLength(5)]
		public string LiabilityInsuranceNaicNumber
		{
			get => _liabilityInsuranceNaicNumber;
			set => Set(ref _liabilityInsuranceNaicNumber, value);
		}

		public bool IsLiabilityInsurancePolicyNumberUnknown
		{
			get => _isLiabilityInsurancePolicyNumberUnknown;
			set => Set(ref _isLiabilityInsurancePolicyNumberUnknown, value);
		}

		[MaxLength(20)]
		public string LiabilityInsurancePolicyNumber
		{
			get => _liabilityInsurancePolicyNumber;
			set => Set(ref _liabilityInsurancePolicyNumber, value);
		}

		public bool IsLiabilityInsurancePolicyExpirationDateUnknown
		{
			get => _isLiabilityInsurancePolicyExpirationDateUnknown;
			set => Set(ref _isLiabilityInsurancePolicyExpirationDateUnknown, value);
		}

		public Date? LiabilityInsurancePolicyExpirationDate
		{
			get => _liabilityInsurancePolicyExpirationDate;
			set => Set(ref _liabilityInsurancePolicyExpirationDate, value);
		}

		public VehicleBodyType BodyType
		{
			get => _bodyType;
			set => Set(ref _bodyType, value);
		}

		[MaxLength(200)]
		public string BodyTypeDescription
		{
			get => _bodyTypeDescription;
			set => Set(ref _bodyTypeDescription, value);
		}

		public WeightRating WeightRating
		{
			get => _weightRating;
			set => Set(ref _weightRating, value);
		}

		/// <summary>
		/// Gets or sets this vehicles HazardousMaterialsPlacardStatus, used by MMUCC field V8.4.
		/// </summary>
		public HazardousMaterialsPlacardStatus HazardousMaterialsPlacardStatus
		{
			get => _hazardousMaterialsPlacardStatus;
			set => Set(ref _hazardousMaterialsPlacardStatus, value);
		}

		public VehicleUsage VehicleUsage
		{
			get => _vehicleUsage;
			set => Set(ref _vehicleUsage, value);
		}

		[MaxLength(200)]
		public string VehicleUsageDescription
		{
			get => _vehicleUsageDescription;
			set => Set(ref _vehicleUsageDescription, value);
		}

		public EmergencyVehicleUsage EmergencyVehicleUsage
		{
			get => _emergencyVehicleUsage;
			set => Set(ref _emergencyVehicleUsage, value);
		}

		public bool IsSpeedLimitUnknown
		{
			get => _isSpeedLimitUnknown;
			set => Set(ref _isSpeedLimitUnknown, value);
		}

		public bool IsSpeedLimitNotApplicable
		{
			get => _isSpeedLimitNotApplicable;
			set => Set(ref _isSpeedLimitNotApplicable, value);
		}

		/// <summary>
        /// Gets or sets the posted/statutory speed limit for this vehicle at the time of the crash.
        /// </summary>
		public int? SpeedLimit
		{
			get => _speedLimit;
			set => Set(ref _speedLimit, value);
		}

		public VehicleTravelDirection TravelDirection
		{
			get => _travelDirection;
			set => Set(ref _travelDirection, value);
		}

		public TrafficwayTravelDirection TrafficwayTravelDirection
		{
			get => _trafficwayTravelDirection;
			set => Set(ref _trafficwayTravelDirection, value);
		}

		public TrafficwayDivision TrafficwayDivision
		{
			get => _trafficwayDivision;
			set => Set(ref _trafficwayDivision, value);
		}

		public TrafficwayBarrierType TrafficwayBarrierType
		{
			get => _trafficwayBarrierType;
			set => Set(ref _trafficwayBarrierType, value);
		}

		[MaxLength(200)]
		public string TrafficwayBarrierTypeDescription
		{
			get => _trafficwayBarrierTypeDescription;
			set => Set(ref _trafficwayBarrierTypeDescription, value);
		}

		public TrafficwayHovLanePresence TrafficwayHovLanePresence
		{
			get => _trafficwayHovLanePresence;
			set => Set(ref _trafficwayHovLanePresence, value);
		}

		public TrafficwayHovLaneRelation TrafficwayHovLaneRelation
		{
			get => _trafficwayHovLaneRelation;
			set => Set(ref _trafficwayHovLaneRelation, value);
		}

		public int? TotalThroughLanes
		{
			get => _totalThroughLanes;
			set => Set(ref _totalThroughLanes, value);
		}

		public int? TotalAuxiliaryLanes
		{
			get => _totalAuxiliaryLanes;
			set => Set(ref _totalAuxiliaryLanes, value);
		}

		public RoadwayAlignment RoadwayAlignment
		{
			get => _roadwayAlignment;
			set => Set(ref _roadwayAlignment, value);
		}

		public RoadwayGrade RoadwayGrade
		{
			get => _roadwayGrade;
			set => Set(ref _roadwayGrade, value);
		}

		[ListCannotContainNull]
		public TrafficControlDeviceTypeList TrafficControlDevicesPresent { get; }

		[MaxLength(200)]
		public string TrafficControlDevicesPresentDescription
		{
			get => _trafficControlDevicesPresentDescription;
			set => Set(ref _trafficControlDevicesPresentDescription, value);
		}

		public TrafficSignalStatus TrafficSignalStatus
		{
			get => _trafficSignalStatus;
			set => Set(ref _trafficSignalStatus, value);
		}

		[ListCannotContainNull] 
		public TrafficControlDeviceTypeList TrafficControlDevicesInoperativeOrMissing { get; }

		[MaxLength(200)]
		public string TrafficControlDevicesInoperativeOrMissingDescription
		{
			get => _trafficControlDevicesInoperativeOrMissingDescription;
			set => Set(ref _trafficControlDevicesInoperativeOrMissingDescription, value);
		}

		public VehicleManeuver Maneuver
		{
			get => _maneuver;
			set => Set(ref _maneuver, value);
		}

		[MaxLength(200)]
		public string ManeuverDescription
		{
			get => _maneuverDescription;
			set => Set(ref _maneuverDescription, value);
		}

		public VehicleManeuverReason ManeuverReason
		{
			get => _maneuverReason;
			set => Set(ref _maneuverReason, value);
		}

		[MaxLength(200)]
		public string ManeuverReasonDescription
		{
			get => _maneuverReasonDescription;
			set => Set(ref _maneuverReasonDescription, value);
		}

		public VehicleLighting Lighting
		{
			get => _lighting;
			set => Set(ref _lighting, value);
		}

		/// <summary>
		/// Gets or sets the initial point of contact for this vehicle, used by MMUCC field V19.1.
		/// </summary>
		public VehicleArea InitialPointOfContact
		{
			get => _initialPointOfContact;
			set => Set(ref _initialPointOfContact, value);
		}

		/// <summary>
		/// Gets a list of the damaged areas on this vehicle, used by MMUCC field V19.2.
		/// </summary>
		[ListCannotContainNull] 
		public VehicleAreaList DamagedAreas { get; }

		public VehicleDamageExtent DamageExtent
		{
			get => _damageExtent;
			set => Set(ref _damageExtent, value);
		}

		[ListCannotContainNull] 
		public SequenceOfEvents SequenceOfEvents { get; }

		/// <summary>
		/// Gets or sets this vehicle or driver's hit and run status, as defined by MMUCC field V22.
		/// </summary>
		public HitAndRun HitAndRun
		{
			get => _hitAndRun;
			set => Set(ref _hitAndRun, value);
		}

		public TowStatus TowStatus
		{
			get => _towStatus;
			set => Set(ref _towStatus, value);
		}

		public bool IsTowServiceUnknown
		{
			get => _isTowServiceUnknown;
			set => Set(ref _isTowServiceUnknown, value);
		}

		[MaxLength(100)]
		public string TowService
		{
			get => _towService;
			set => Set(ref _towService, value);
		}

		public TowAuthority TowAuthority
		{
			get => _towAuthority;
			set => Set(ref _towAuthority, value);
		}

		[MaxLength(200)]
		public string TowAuthorityDescription
		{
			get => _towAuthorityDescription;
			set => Set(ref _towAuthorityDescription, value);
		}

		public VehicleContributingCircumstance ContributingCircumstance
		{
			get => _contributingCircumstance;
			set => Set(ref _contributingCircumstance, value);
		}

		[MaxLength(200)]
		public string ContributingCircumstanceDescription
		{
			get => _contributingCircumstanceDescription;
			set => Set(ref _contributingCircumstanceDescription, value);
		}

		[JsonSchemaComment("If the vehicle had no driver, set Driver to null.")]
		public Driver Driver
		{
			get => _driver;
			set
			{
				Set(ref _driver, value);
			}
		}

		[ListCannotContainNull] 
		public PassengerList Passengers { get; }

		public VehicleFmcsaReportability FmcsaReportability
		{
			get => _fmcsaReportability;
			set => Set(ref _fmcsaReportability, value);
		}

		public bool IsNumberOfAxlesUnknown
		{
			get => _isNumberOfAxlesUnknown;
			set => Set(ref _isNumberOfAxlesUnknown, value);
		}

		/// <summary>
		/// Gets or sets the number of axles on this vehicle.
		/// </summary>
		public int? NumberOfAxles
		{
			get => _numberOfAxles;
			set => Set(ref _numberOfAxles, value);
		}

		/// <summary>
		/// Gets the list of trailers attached to this vehicle.
		/// </summary>
		[ListCannotContainNull]
		public VehicleTrailerList Trailers { get; }

		public MotorCarrierIdentificationType MotorCarrierIdentificationType
		{
			get => _motorCarrierIdentificationType;
			set => Set(ref _motorCarrierIdentificationType, value);
		}

		public State MotorCarrierIdentificationState
		{
			get => _motorCarrierIdentificationState;
			set => Set(ref _motorCarrierIdentificationState, value);
		}

		[MaxLength(200)]
		public string MotorCarrierIdentificationStateDescription
		{
			get => _motorCarrierIdentificationStateDescription;
			set => Set(ref _motorCarrierIdentificationStateDescription, value);
		}

		[MaxLength(25)]
		public string MotorCarrierIdentificationNumber
		{
			get => _motorCarrierIdentificationNumber;
			set => Set(ref _motorCarrierIdentificationNumber, value);
		}

		public bool IsMotorCarrierNameUnknown
		{
			get => _isMotorCarrierNameUnknown;
			set => Set(ref _isMotorCarrierNameUnknown, value);
		}

		/// <summary>
		/// Gets or sets the name of this vehicle's motor carrier.
		/// </summary>
		[MaxLength(100)]
		public string MotorCarrierName
		{
			get => _motorCarrierName;
			set => Set(ref _motorCarrierName, value);
		}

		public bool IsMotorCarrierAddressUnknown
		{
			get => _isMotorCarrierAddressUnknown;
			set => Set(ref _isMotorCarrierAddressUnknown, value);
		}

		[MaxLength(100)]
		public string MotorCarrierAddressStreet
		{
			get => _motorCarrierAddressStreet;
			set => Set(ref _motorCarrierAddressStreet, value);
		}

		[MaxLength(50)]
		public string MotorCarrierAddressCity
		{
			get => _motorCarrierAddressCity;
			set => Set(ref _motorCarrierAddressCity, value);
		}

		public State MotorCarrierAddressState
		{
			get => _motorCarrierAddressState;
			set => Set(ref _motorCarrierAddressState, value);
		}

		[MaxLength(200)]
		public string MotorCarrierAddressStateDescription
		{
			get => _motorCarrierAddressStateDescription;
			set => Set(ref _motorCarrierAddressStateDescription, value);
		}

		[MaxLength(20)]
		public string MotorCarrierAddressPostalCode
		{
			get => _motorCarrierAddressPostalCode;
			set => Set(ref _motorCarrierAddressPostalCode, value);
		}

		public bool IsMotorCarrierPhoneNumberUnknown
		{
			get => _isMotorCarrierPhoneNumberUnknown;
			set => Set(ref _isMotorCarrierPhoneNumberUnknown, value);
		}

		[MaxLength(20)]
		public string MotorCarrierPhoneNumber
		{
			get => _motorCarrierPhoneNumber;
			set => Set(ref _motorCarrierPhoneNumber, value);
		}

		public MotorCarrierType MotorCarrierType
		{
			get => _motorCarrierType;
			set => Set(ref _motorCarrierType, value);
		}

		[MaxLength(200)]
		public string MotorCarrierTypeDescription
		{
			get => _motorCarrierTypeDescription;
			set => Set(ref _motorCarrierTypeDescription, value);
		}

		public VehicleConfiguration VehicleConfiguration
		{
			get => _vehicleConfiguration;
			set => Set(ref _vehicleConfiguration, value);
		}

		[ListCannotContainNull]
		public VehicleSpecialSizingList VehicleSpecialSizings { get; }

		public VehicleLoadPermit VehicleLoadPermit
		{
			get => _vehicleLoadPermit;
			set => Set(ref _vehicleLoadPermit, value);
		}

		public CargoBodyType CargoBodyType
		{
			get => _cargoBodyType;
			set => Set(ref _cargoBodyType, value);
		}

		[MaxLength(200)]
		public string CargoBodyTypeDescription
		{
			get => _cargoBodyTypeDescription;
			set => Set(ref _cargoBodyTypeDescription, value);
		}

		public bool IsCommodityHauledUnknown
		{
			get => _isCommodityHauledUnknown;
			set => Set(ref _isCommodityHauledUnknown, value);
		}

		[MaxLength(100)]
		public string CommodityHauled
		{
			get => _commodityHauled;
			set => Set(ref _commodityHauled, value);
		}

		public bool IsHazardousMaterialsIdUnknown
		{
			get => _isHazardousMaterialsIdUnknown;
			set => Set(ref _isHazardousMaterialsIdUnknown, value);
		}

		public bool IsHazardousMaterialsIdNotApplicable
		{
			get => _isHazardousMaterialsIdNotApplicable;
			set => Set(ref _isHazardousMaterialsIdNotApplicable, value);
		}

		/// <summary>
		/// Gets or sets the ID of hazardous materials being transported by this vehicle, as defined by MMUCC field LV10.1.
		/// </summary>
		[MaxLength(20)]
		public string HazardousMaterialsId
		{
			get => _hazardousMaterialsId;
			set => Set(ref _hazardousMaterialsId, value);
		}

		/// <summary>
		/// Gets or sets the class of hazardous materials being transported by this vehicle, as defined by MMUCC field LV10.2.
		/// </summary>
		public HazardousMaterialClass HazardousMaterialClass
		{
			get => _hazardousMaterialClass;
			set => Set(ref _hazardousMaterialClass, value);
		}

		/// <summary>
		/// Gets or sets whether or not the hazardous materials being transported by this vehicle were released, as defined by MMUCC field LV10.3.
		/// </summary>
		public HazardousMaterialRelease HazardousMaterialRelease
		{
			get => _hazardousMaterialRelease;
			set => Set(ref _hazardousMaterialRelease, value);
		}

		/// <summary>
		/// Gets or sets the level of automation systems present in the vehicle, as defined by MMUCC field DV1.2.
		/// </summary>
		public VehicleAutomationSystemLevel AutomationSystemLevelPresent
		{
			get => _automationSystemLevelPresent;
			set => Set(ref _automationSystemLevelPresent, value);
		}

		/// <summary>
		/// Gets or sets the level of automation systems engaged in the vehicle at the time of the crash, as defined by MMUCC field DV1.3.
		/// </summary>
		public VehicleAutomationSystemLevel AutomationSystemLevelEngaged
		{
			get => _automationSystemLevelEngaged;
			set => Set(ref _automationSystemLevelEngaged, value);
		}

		public bool IsDistanceTraveledAfterImpactUnknown
		{
			get => _isDistanceTraveledAfterImpactUnknown;
			set => Set(ref _isDistanceTraveledAfterImpactUnknown, value);
		}

		public int? DistanceTraveledAfterImpact
		{
			get => _distanceTraveledAfterImpact;
			set => Set(ref _distanceTraveledAfterImpact, value);
		}

		public bool IsSkidmarkLengthNotApplicable
		{
			get => _isSkidmarkLengthNotApplicable;
			set => Set(ref _isSkidmarkLengthNotApplicable, value);
		}

		public bool IsSkidmarkLengthUnknown
		{
			get => _isSkidmarkLengthUnknown;
			set => Set(ref _isSkidmarkLengthUnknown, value);
		}

		public int? SkidmarkLengthFrontLeft
		{
			get => _skidmarkLengthFrontLeft;
			set => Set(ref _skidmarkLengthFrontLeft, value);
		}

		public int? SkidmarkLengthFrontRight
		{
			get => _skidmarkLengthFrontRight;
			set => Set(ref _skidmarkLengthFrontRight, value);
		}

		public int? SkidmarkLengthRearLeft
		{
			get => _skidmarkLengthRearLeft;
			set => Set(ref _skidmarkLengthRearLeft, value);
		}

		public int? SkidmarkLengthRearRight
		{
			get => _skidmarkLengthRearRight;
			set => Set(ref _skidmarkLengthRearRight, value);
		}

		public LicensePlateType LicensePlateType
		{
			get => _licensePlateType;
			set => Set(ref _licensePlateType, value);
		}
	}
}