﻿using System.Collections.Generic;
using ECrash.Core.Models.Document;
using ECrashLA.Core.Models.CrashReports.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	public class VehicleTrailer : DocumentChild<CrashReport>, INeedsCollectionIndex, IHasVehicleRegistration
	{
		private bool _isVinUnknown;
		private string _vin;
		private bool _isTagMissing;
		private bool _isTagStateUnknown;
		private State _tagState;
		private string _tagStateDescription;
		private bool _isTagNumberUnknown;
		private string _tagNumber;
		private bool _isTagNonExpiring;
		private bool _isTagYearUnknown;
		private int? _tagYear;
		private bool _isMakeUnknown;
		private string _make;
		private bool _isModelUnknown;
		private string _model;
		private bool _isModelYearUnknown;
		private int? _modelYear;
		private int? _numberOfAxles;
		private bool _isNumberOfAxlesUnknown;

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			yield break;
		}

		[JsonIgnore]
		public int Index => (Parent as VehicleTrailerList)?.IndexOf(this) ?? -1;

		[JsonIgnore]
		public int Number => Index + 1;

		public bool IsVinUnknown
		{
			get => _isVinUnknown;
			set => Set(ref _isVinUnknown, value);
		}

		[MaxLength(20)]
		public string Vin
		{
			get => _vin;
			set => Set(ref _vin, value);
		}

		public bool IsTagMissing
		{
			get => _isTagMissing;
			set => Set(ref _isTagMissing, value);
		}

		public bool IsTagStateUnknown
		{
			get => _isTagStateUnknown;
			set => Set(ref _isTagStateUnknown, value);
		}

		public State TagState
		{
			get => _tagState;
			set => Set(ref _tagState, value);
		}

		[MaxLength(200)]
		public string TagStateDescription
		{
			get => _tagStateDescription;
			set => Set(ref _tagStateDescription, value);
		}

		public bool IsTagNumberUnknown
		{
			get => _isTagNumberUnknown;
			set => Set(ref _isTagNumberUnknown, value);
		}

		[MaxLength(10)]
		public string TagNumber
		{
			get => _tagNumber;
			set => Set(ref _tagNumber, value);
		}

		public bool IsTagNonExpiring
		{
			get => _isTagNonExpiring;
			set => Set(ref _isTagNonExpiring, value);
		}

		public bool IsTagYearUnknown
		{
			get => _isTagYearUnknown;
			set => Set(ref _isTagYearUnknown, value);
		}

		public int? TagYear
		{
			get => _tagYear;
			set => Set(ref _tagYear, value);
		}

		public bool IsMakeUnknown
		{
			get => _isMakeUnknown;
			set => Set(ref _isMakeUnknown, value);
		}

		[MaxLength(50)]
		public string Make
		{
			get => _make;
			set => Set(ref _make, value);
		}

		public bool IsModelUnknown
		{
			get => _isModelUnknown;
			set => Set(ref _isModelUnknown, value);
		}

		[MaxLength(50)]
		public string Model
		{
			get => _model;
			set => Set(ref _model, value);
		}

		public bool IsModelYearUnknown
		{
			get => _isModelYearUnknown;
			set => Set(ref _isModelYearUnknown, value);
		}

		public int? ModelYear
		{
			get => _modelYear;
			set => Set(ref _modelYear, value);
		}

		public int? NumberOfAxles
		{
			get => _numberOfAxles;
			set => Set(ref _numberOfAxles, value);
		}

		public bool IsNumberOfAxlesUnknown
		{
			get => _isNumberOfAxlesUnknown;
			set => Set(ref _isNumberOfAxlesUnknown, value);
		}

		void INeedsCollectionIndex.OnCollectionIndexChanged()
		{
			NotifyOfPropertyChange(nameof(Index));
			NotifyOfPropertyChange(nameof(Number));
		}
	}
}