﻿using System.Collections.Generic;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.Utilities;
using ECrashLA.Core.Models.CrashReports.ReferenceData;
using Newtonsoft.Json;

namespace ECrashLA.Core.Models.CrashReports
{
	public class Witness : DocumentChild<CrashReport>, INeedsCollectionIndex
	{
		private string _lastName;
		private string _firstName;
		private string _middleName;
		private string _nameSuffix;
		private bool _isAgeUnknown;
		private int? _age;
		private Sex _sex;
		private bool _isAddressUnknown;
		private string _addressStreet;
		private string _addressCity;
		private State _addressState;
		private string _addressStateDescription;
		private string _addressPostalCode;
		private string _phoneNumber;

		public override IEnumerable<IDocumentNode<CrashReport>> EnumerateChildNodes()
		{
			yield break;
		}

		/// <summary>
		/// Gets or sets the zero-based index of this witness.
		/// </summary>
		[JsonIgnore]
		public int Index => (Parent as WitnessList)?.IndexOf(this) ?? -1;

		/// <summary>
		/// Gets the one-based number of this witness.
		/// </summary>
		[JsonIgnore]
		public int Number => Index + 1;

		[MaxLength(50)]
		public string LastName
		{
			get => _lastName;
			set => SetWithDependencies(ref _lastName, value, nameof(LastName), nameof(FullName));
		}

		[MaxLength(50)]
		public string FirstName
		{
			get => _firstName;
			set => SetWithDependencies(ref _firstName, value, nameof(FirstName), nameof(FullName));
		}

		[MaxLength(50)]
		public string MiddleName
		{
			get => _middleName;
			set => SetWithDependencies(ref _middleName, value, nameof(MiddleName), nameof(FullName));
		}

		[MaxLength(10)]
		public string NameSuffix
		{
			get => _nameSuffix;
			set => SetWithDependencies(ref _nameSuffix, value, nameof(NameSuffix), nameof(FullName));
		}

		[JsonIgnore]
		public string FullName => GetFullName(PersonNameOrder.FirstNameFirst);

		public bool IsAgeUnknown
		{
			get => _isAgeUnknown;
			set => Set(ref _isAgeUnknown, value);
		}

		public int? Age
		{
			get => _age;
			set => Set(ref _age, value);
		}

		public Sex Sex
		{
			get => _sex;
			set => Set(ref _sex, value);
		}

		public bool IsAddressUnknown
		{
			get => _isAddressUnknown;
			set => Set(ref _isAddressUnknown, value);
		}

		[MaxLength(150)]
		public string AddressStreet
		{
			get => _addressStreet;
			set => Set(ref _addressStreet, value);
		}

		[MaxLength(50)]
		public string AddressCity
		{
			get => _addressCity;
			set => Set(ref _addressCity, value);
		}

		public State AddressState
		{
			get => _addressState;
			set => Set(ref _addressState, value);
		}

		[MaxLength(200)]
		public string AddressStateDescription
		{
			get => _addressStateDescription;
			set => Set(ref _addressStateDescription, value);
		}

		[MaxLength(20)]
		public string AddressPostalCode
		{
			get => _addressPostalCode;
			set => Set(ref _addressPostalCode, value);
		}

		[MaxLength(20)]
		public string PhoneNumber
		{
			get => _phoneNumber;
			set => Set(ref _phoneNumber, value);
		}

		void INeedsCollectionIndex.OnCollectionIndexChanged()
		{
			NotifyOfPropertyChange(nameof(Index));
			NotifyOfPropertyChange(nameof(Number));
		}

		public string GetFullName(PersonNameOrder nameOrder)
		{
			return NameUtilities.GetFullName(nameOrder, _firstName, _middleName, _lastName, _nameSuffix);
		}
	}
}