﻿using System;
using System.Net;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using ECrash.Core.Models.ReferenceData;
using ECrashLA.Console.Settings;
using Flurl;
using Flurl.Http;
using Newtonsoft.Json;
using Spectre.Console;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Commands
{
	public class GetReferenceDataCommand : AsyncCommand<ReferenceDataSettings>
	{
		private static readonly Regex EtagRegex = new Regex(@"^(?:W/)?\""(.*)\""$");

		public override async Task<int> ExecuteAsync(CommandContext context, ReferenceDataSettings settings)
		{
			try
			{
				var request = settings.Url
				                      .AppendPathSegment("public/referencedata/v1")
				                      .AllowHttpStatus((int)HttpStatusCode.NotModified);
				if (settings.ETag != null)
				{
					request = request.WithHeader("If-None-Match", $"\"{settings.ETag}\"");
				}
				var response = await request.GetAsync();
				if (response.StatusCode == (int)HttpStatusCode.NotModified)
				{
					return response.StatusCode;
				}
				var referenceData = await response.GetJsonAsync<ReferenceDataList>();
				string etag = response.Headers.FirstOrDefault("ETag");
				if (etag != null)
				{
					etag = EtagRegex.Replace(etag, @"$1");
				}

				string json = JsonConvert.SerializeObject(new
				{
					Etag = etag,
					ReferenceData = referenceData
				}, settings.Indent ? Formatting.Indented : Formatting.None);
				await System.Console.Out.WriteLineAsync(json);
				return 0;
			}
			catch (FlurlHttpException ex)
			{
				AnsiConsole.WriteException(ex);
				return ex.StatusCode ?? -1;
			}
			catch (Exception ex)
			{
				AnsiConsole.WriteException(ex);
				return -1;
			}
		}
	}
}