﻿using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Threading.Tasks;
using Castle.Core.Logging;
using ECrashLA.Console.Settings;
using Flurl;
using Flurl.Http;
using Flurl.Http.Content;
using Spectre.Console;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Commands
{
	public class ModernizeDiagramCommand : VendorCommandBase<ModernizeDiagramSettings>
	{
		public ModernizeDiagramCommand(ILogger logger) : base(logger)
		{
		}
		
		protected override async Task<int> ExecuteAsyncCore(CommandContext context, ModernizeDiagramSettings settings, string token)
		{
			Directory.CreateDirectory(settings.OutputPath);
			var svgFiles = Directory.EnumerateFiles(settings.OutputPath, "*.svg").ToList();
			if (svgFiles.Count > 0)
			{
				if (!AnsiConsole.Confirm("The directory already contains one or more SVG files. Delete these and proceed?", false))
				{
					await System.Console.Out.WriteLineAsync("Operation canceled.");
					return 1;
				}
				foreach (string file in svgFiles)
				{
					File.Delete(file);
				}
			}
			await using var zipStream = await settings.Url.AppendPathSegment("vendor/diagram/v1/modernize")
			                                          .WithOAuthBearerToken(token)
			                                          .PostAsync(new FileContent(settings.DiagramPath))
			                                          .ReceiveStream();
			using var zipFile = new ZipArchive(zipStream);
			foreach (var entry in zipFile.Entries)
			{
				await using var entryStream = entry.Open();
				await using var fileStream = File.Create(Path.Combine(settings.OutputPath, entry.Name));
				await entryStream.CopyToAsync(fileStream);
			}
			return 0;
		}
	}
}