﻿using System.IO;
using System.Threading.Tasks;
using Castle.Core.Logging;
using ECrashLA.Console.Settings;
using Flurl;
using Flurl.Http;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Commands
{
	public class VendorDownloadAttachmentCommand : VendorCommandBase<VendorDownloadAttachmentSettings>
	{
		public VendorDownloadAttachmentCommand(ILogger logger) : base(logger)
		{
		}

		protected override async Task<int> ExecuteAsyncCore(CommandContext context, VendorDownloadAttachmentSettings settings, string token)
		{
			string outputPath = settings.OutputPath;
			if (outputPath == null)
			{
				outputPath = settings.Filename;
			}
			else if (Directory.Exists(outputPath))
			{
				outputPath = Path.Combine(outputPath, settings.Filename);
			}
			Logger.Debug($"Downloading attachment {settings.Filename} for crash report {settings.Id}...");
			await using var inputStream = await settings.Url.AppendPathSegments("vendor/download/v1/attachment", settings.Id, settings.Hash, settings.Filename)
			                                            .WithOAuthBearerToken(token)
			                                            .GetStreamAsync();
			await using var outputStream = File.Create(outputPath);
			await inputStream.CopyToAsync(outputStream);
			return 0;
		}
	}
}