﻿using System.IO;
using System.Threading.Tasks;
using Castle.Core.Logging;
using ECrashLA.Console.Settings;
using Flurl;
using Flurl.Http;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Commands
{
	public class VendorDownloadCrashReportPdfCommand : VendorCommandBase<VendorDownloadCrashReportPdfSettings>
	{
		public VendorDownloadCrashReportPdfCommand(ILogger logger) : base(logger)
		{
		}

		protected override async Task<int> ExecuteAsyncCore(CommandContext context, VendorDownloadCrashReportPdfSettings settings, string token)
		{
			string outputPath = settings.OutputPath ?? ".";
			if (Directory.Exists(outputPath))
			{
				outputPath = Path.Combine(outputPath, $"{settings.Id}.pdf");
			}
			Logger.Debug($"Downloading PDF for crash report {settings.Id}...");
			try
			{
				await using var inputStream = await settings.Url.AppendPathSegments("vendor/download/v1/pdf", settings.Id)
				                                            .SetQueryParam("revision", settings.Revision)
				                                            .SetQueryParam("includeAttachments", !settings.OmitAttachments)
				                                            .SetQueryParam("includePhotos", !settings.OmitPhotos)
				                                            .SetQueryParam("redact", settings.Redact)
				                                            .WithOAuthBearerToken(token)
				                                            .GetStreamAsync();
				await using var outputStream = File.Create(outputPath);
				await inputStream.CopyToAsync(outputStream);
				return 0;
			}
			catch (FlurlHttpException ex) when (ex.StatusCode == 451)
			{
				Logger.Error($"Crash report {settings.Id} is not approved and you do not have permission to download it");
				return 451;
			}
		}
	}
}