﻿using System;
using System.Threading.Tasks;
using Castle.Core.Logging;
using ECrash.Core;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.Document.Upload;
using ECrashLA.Console.Services.Serialization;
using ECrashLA.Console.Settings;
using ECrashLA.Core.Models.CrashReports;
using Newtonsoft.Json;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Commands
{
	public class VendorUploadCrashReportCommand : VendorCommandBase<VendorCrashReportUploadSettings>
	{
		private readonly ICrashReportSerializer _serializer;
		private readonly IDocumentUploader<CrashReport, CrashReportStatus> _uploader;

		public VendorUploadCrashReportCommand(ICrashReportSerializer serializer, 
		                                      ILogger logger, 
		                                      IDocumentUploader<CrashReport, CrashReportStatus> uploader)
			: base(logger)
		{
			_serializer = serializer;
			_uploader = uploader;
		}

		protected override async Task<int> ExecuteAsyncCore(CommandContext context, VendorCrashReportUploadSettings settings, string token)
		{
			Logger.Info("Loading crash report...");
			var crashReport = await _serializer.LoadAsync(settings.CrashReportPath);
			
			Logger.Info("Uploading crash report data...");
			var uploadContext = new ContextDictionary
			{
				["token"] = token, 
				["url"] = settings.Url,
				["reviewer"] = new Actor(
					Guid.Empty,
					string.Empty,
					settings.ReviewingOfficerRank,
					settings.ReviewingOfficerFirstName,
					settings.ReviewingOfficerMiddleName,
					settings.ReviewingOfficerLastName,
					settings.ReviewingOfficerNameSuffix,
					settings.ReviewingOfficerId)
			};
			var result = await _uploader.UploadDocumentAsync(crashReport, DocumentUploadMode.Upload, uploadContext);
			await System.Console.Out.WriteLineAsync(JsonConvert.SerializeObject(result));
			if (result.Succeeded)
			{
				return 0;
			}
			if (result.Exception != null)
			{
				return result.Exception.HResult;
			}
			return -1;
		}
	}
}