﻿using System.Threading.Tasks;
using Castle.Core.Logging;
using ECrash.Core;
using ECrash.Core.Models.Document.Upload;
using ECrashLA.Console.Services.Rendering.Validation;
using ECrashLA.Console.Services.Serialization;
using ECrashLA.Console.Settings;
using ECrashLA.Core.Models.CrashReports;
using Newtonsoft.Json;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Commands
{
	public class VendorValidateCrashReportCommand : VendorCommandBase<VendorValidateCrashReportSettings>
	{
		private readonly ICrashReportSerializer _serializer;
		private readonly IDocumentUploader<CrashReport, CrashReportStatus> _uploader;
		private readonly IValidationReportRenderer _renderer;

		public VendorValidateCrashReportCommand(ICrashReportSerializer serializer,
		                                        ILogger logger,
		                                        IDocumentUploader<CrashReport, CrashReportStatus> uploader,
		                                        IValidationReportRenderer renderer)
			: base(logger)
		{
			_serializer = serializer;
			_uploader = uploader;
			_renderer = renderer;
		}

		protected override async Task<int> ExecuteAsyncCore(CommandContext context, VendorValidateCrashReportSettings settings, string token)
		{
			Logger.Info("Loading crash report...");
			var crashReport = await _serializer.LoadAsync(settings.CrashReportPath);

			Logger.Info("Uploading crash report data...");
			var uploadContext = new ContextDictionary
			{
				["token"] = token,
				["url"] = settings.Url
			};
			var result = await _uploader.UploadDocumentAsync(crashReport, DocumentUploadMode.Validate, uploadContext);
			await _renderer.RenderAsync(result.ValidationReport, settings.Format);
			if (result.Succeeded)
			{
				return 0;
			}
			if (result.Exception != null)
			{
				return result.Exception.HResult;
			}
			return -1;
		}
	}
}