﻿using System;
using System.Collections.Generic;
using Castle.MicroKernel.Registration;
using Castle.MicroKernel.SubSystems.Configuration;
using Castle.Windsor;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Ioc
{
	public class WindsorTypeRegistrar : IWindsorInstaller, ITypeRegistrar, ITypeResolver
	{
		private readonly IWindsorContainer _container;
		private readonly List<IRegistration> _registrations = new();
		private readonly HashSet<object> _implementations = new();

		public WindsorTypeRegistrar(IWindsorContainer container)
		{
			_container = container;
		}

		public void Register(Type service, Type implementation)
		{
			if (_implementations.Add(implementation))
			{
				_registrations.Add(Component.For(service, implementation).ImplementedBy(implementation).LifestyleSingleton());
			}
		}

		public void RegisterInstance(Type service, object implementation)
		{
			if (_implementations.Add(implementation))
			{
				_registrations.Add(Component.For(service).Instance(implementation));
			}
		}

		public void RegisterLazy(Type service, Func<object> factory)
		{
			if (_implementations.Add(factory))
			{
				_registrations.Add(Component.For(service).UsingFactoryMethod(factory));
			}
		}

		public ITypeResolver Build()
		{
			_container.Install(this);
			return this;
		}

		public object Resolve(Type type)
		{
			return _container.Kernel.HasComponent(type) ? _container.Resolve(type) : null;
		}

		public void Install(IWindsorContainer container, IConfigurationStore store)
		{
			container.Register(_registrations.ToArray());
		}
	}
}