﻿using System.Threading.Tasks;
using Castle.Core.Logging;
using ECrash.Core;
using ECrash.Core.Ioc;
using ECrash.Core.Models.Document;
using ECrash.Core.Models.Document.Upload;
using ECrashLA.Core.Models.CrashReports;
using Flurl;
using Flurl.Http;

namespace ECrashLA.Console.Models.CrashReports.Upload
{
	[Singleton]
	[Service(typeof(IDocumentUploader<CrashReport, CrashReportStatus>))]
	public class VendorCrashReportUploader : DocumentUploaderBase<CrashReport, CrashReportStatus>
	{
		public VendorCrashReportUploader(ILogger logger) : base(logger)
		{
		}

		protected override Task PreAuthenticateAsync(ContextDictionary context)
		{
			return Task.CompletedTask;
		}

		protected override IFlurlRequest CreateDocumentUploadRequest(CrashReport document, DocumentUploadMode mode, ContextDictionary context)
		{
			string url = (string)context["url"];
			var reviewer = context.TryGetValue("reviewer", out object obj) ? obj as Actor : null;
			return new FlurlRequest(url.AppendPathSegment("vendor/upload/v1/crashReport")
			                           .SetQueryParam("mode", mode)
			                           .SetQueryParam("reviewingOfficerRank", reviewer?.Rank)
			                           .SetQueryParam("reviewingOfficerFirstName", reviewer?.FirstName)
			                           .SetQueryParam("reviewingOfficerMiddleName", reviewer?.MiddleName)
			                           .SetQueryParam("reviewingOfficerLastName", reviewer?.LastName)
			                           .SetQueryParam("reviewingOfficerNameSuffix", reviewer?.NameSuffix)
			                           .SetQueryParam("reviewingOfficerId", reviewer?.OfficerId));
		}

		protected override IFlurlRequest CreateAttachmentUploadRequest(CrashReport document, string token, AttachmentBase<CrashReport> attachment, ContextDictionary context)
		{
			string url = (string)context["url"];
			return new FlurlRequest(url.AppendPathSegment("vendor/upload/v1/attachment")
			                           .AppendPathSegment(document.Id)
			                           .AppendPathSegment(token));
		}

		protected override IFlurlRequest CreateDocumentCommitRequest(CrashReport document, string token, DocumentUploadMode mode, ContextDictionary context)
		{
			string url = (string)context["url"];
			var reviewer = context.TryGetValue("reviewer", out object obj) ? obj as Actor : null;
			return new FlurlRequest(url.AppendPathSegment("vendor/upload/v1/commit")
			                           .AppendPathSegment(document.Id)
			                           .AppendPathSegment(token)
			                           .SetQueryParam("mode", mode)
									   .SetQueryParam("reviewingOfficerRank", reviewer?.Rank)
			                           .SetQueryParam("reviewingOfficerFirstName", reviewer?.FirstName)
			                           .SetQueryParam("reviewingOfficerMiddleName", reviewer?.MiddleName)
			                           .SetQueryParam("reviewingOfficerLastName", reviewer?.LastName)
			                           .SetQueryParam("reviewingOfficerNameSuffix", reviewer?.NameSuffix)
			                           .SetQueryParam("reviewingOfficerId", reviewer?.OfficerId));
		}

		protected override IFlurlRequest AuthenticateRequest(IFlurlRequest request, ContextDictionary context)
		{
			return request.WithOAuthBearerToken((string)context["token"]);
		}

		protected override Task<UploadResult<CrashReportStatus>> OnUploadCompleted(CrashReport document, CompleteDocumentUploadResult<CrashReportStatus> completedUpload, ContextDictionary context)
		{
			return Task.FromResult(new UploadResult<CrashReportStatus>(completedUpload.Status, completedUpload.ValidationReport));
		}
	}
}