﻿using System.Threading.Tasks;
using Castle.Core.Logging;
using Castle.Windsor;
using ECrashLA.Console.Commands;
using ECrashLA.Console.Ioc;
using Flurl.Http;
using Spectre.Console.Cli;

namespace ECrashLA.Console
{
	class Program
	{
		public static readonly IWindsorContainer Container = new WindsorContainer();
		private static ILogger _logger;

		static async Task<int> Main(string[] args)
		{
			Container.Install(new ConsoleInstaller());
			var loggerFactory = Container.Resolve<ILoggerFactory>();
			_logger = loggerFactory.Create(typeof(Program));

			var app = new CommandApp(new WindsorTypeRegistrar(Container));
			app.Configure(config =>
			{
				config.AddBranch("refdata",
				                 refdata =>
				                 {
					                 refdata.AddCommand<GetReferenceDataCommand>("get");
				                 });
				config.AddBranch("vendor",
				                 vendor =>
				                 {
					                 vendor.AddCommand<LoginVendorCommand>("login");
					                 vendor.AddCommand<VendorUploadCrashReportCommand>("upload");
					                 vendor.AddCommand<VendorValidateCrashReportCommand>("validate");
					                 vendor.AddBranch("download",
					                                       download =>
					                                       {
						                                       download.AddCommand<VendorDownloadCrashReportStatusesCommand>("statuses");
						                                       download.AddCommand<VendorDownloadCrashReportsCommand>("reports");
						                                       download.AddCommand<VendorDownloadCrashReportCommand>("report");
						                                       download.AddCommand<VendorDownloadAttachmentCommand>("attachment");
						                                       download.AddCommand<VendorDownloadCrashReportPdfCommand>("pdf");
						                                       download.AddCommand<VendorDownloadFilteredCrashReportDataCommand>("filteredReports");
					                                       });
									 vendor.AddBranch("diagram",
					                                  diagram =>
					                                  {
						                                  diagram.AddCommand<ModernizeDiagramCommand>("modernize");
					                                  });
				                 });
				config.AddBranch("crashReport", crashReport =>
				{
					crashReport.AddCommand<LocalValidateCrashReportCommand>("validate");
				});
				config.Settings.CaseSensitivity = CaseSensitivity.None;
			});
			try
			{
				return await app.RunAsync(args);
			}
			catch (FlurlHttpException ex) when (ex.StatusCode != null)
			{
				_logger.Error($"Operation failed with HTTP status code {ex.StatusCode}", ex);
				return ex.StatusCode.Value;
			}
		}
	}
}
