﻿using System.ComponentModel;
using System.Text.RegularExpressions;
using Spectre.Console;
using Spectre.Console.Cli;

namespace ECrashLA.Console.Settings
{
	public class ReferenceDataSettings : WebApiSettings
	{
		public static readonly Regex EtagRegex = new Regex("^[0-9a-fA-F]{16}$");

		[CommandOption("-i|--indent")]
		[Description("Indents the JSON output.")]
		public bool Indent { get; set; }

		[CommandOption("--etag <etag>")]
		[Description("The ETag to use when checking for updated reference data.")]
		public string ETag { get; set; }

		public override ValidationResult Validate()
		{
			var result = base.Validate();
			if (result.Successful && ETag != null && !EtagRegex.IsMatch(ETag))
			{
				result = ValidationResult.Error("etag parameter should be 16 hexadecimal digits");
			}
			return result;
		}
	}
}